@@ -16,7 +16,7 @@ from tortoisehg.util import hglib
 from tortoisehg.util.patchctx import patchctx
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, qscilib, lexers, visdiff, revert, rejects
-from tortoisehg.hgqt import filelistmodel, filelistview, filedata, blockmatcher
+from tortoisehg.hgqt import filelistview, filedata, blockmatcher, manifestmodel
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -38,11 +38,10 @@ class ChunksWidget(QWidget):
 
     contextmenu = None
 
-    def __init__(self, repoagent, parent, multiselectable):
+    def __init__(self, repoagent, parent):
         QWidget.__init__(self, parent)
 
         self._repoagent = repoagent
-        self.multiselectable = multiselectable
         self.currentFile = None
 
         layout = QVBoxLayout(self)
@@ -57,9 +56,10 @@ class ChunksWidget(QWidget):
         self.layout().addWidget(self.splitter)
 
         repo = self._repoagent.rawRepo()
-        self.filelist = filelistview.HgFileListView(repo, self, multiselectable)
-        self.filelistmodel = filelistmodel.HgFileListModel(self)
-        self.filelist.setModel(self.filelistmodel)
+        self.filelist = filelistview.HgFileListView(self)
+        model = manifestmodel.ManifestModel(
+            repoagent, self, statusfilter='MAR', flat=True)
+        self.filelist.setModel(model)
         self.filelist.setContextMenuPolicy(Qt.CustomContextMenu)
         self.filelist.customContextMenuRequested.connect(self.menuRequest)
         self.filelist.doubleClicked.connect(self.vdiff)
@@ -166,11 +166,12 @@ class ChunksWidget(QWidget):
             pass
 
     def runPatcher(self, fp, wfile, updatestate):
-        ui = self.repo.ui.copy()
+        # don't repo.ui.copy(), which is protected to clone baseui since hg 2.9
+        ui = self.repo.ui
         class warncapt(ui.__class__):
             def warn(self, msg, *args, **opts):
                 self.write(msg)
-        ui.__class__ = warncapt
+        ui = warncapt(ui)
 
         ok = True
         repo = self.repo
@@ -448,7 +449,7 @@ class ChunksWidget(QWidget):
 
     def setContext(self, ctx):
         self.diffbrowse.setContext(ctx)
-        self.filelist.setContext(ctx)
+        self.filelist.model().setRawContext(ctx)
         empty = len(ctx.files()) == 0
         self.fileModelEmpty.emit(empty)
         self.fileSelected.emit(not empty)
@@ -530,10 +531,11 @@ class DiffBrowser(QFrame):
         w.setTextInteractionFlags(f | Qt.TextSelectableByMouse)
         w.linkActivated.connect(self.linkActivated)
 
-        self.searchbar = qscilib.SearchToolBar(hidable=True)
+        self.searchbar = qscilib.SearchToolBar()
         self.searchbar.hide()
         self.searchbar.searchRequested.connect(self.find)
         self.searchbar.conditionChanged.connect(self.highlightText)
+        self.addActions(self.searchbar.editorActions())
 
         guifont = qtlib.getfont('fontlist').font()
         self.sumlabel = QLabel()
@@ -553,7 +555,7 @@ class DiffBrowser(QFrame):
         self.actionFind.setToolTip(_('Toggle display of text search bar'))
         qtlib.newshortcutsforstdkey(QKeySequence.Find, self, self.searchbar.show)
         self.diffToolbar = QToolBar(_('Diff Toolbar'))
-        self.diffToolbar.setIconSize(QSize(16, 16))
+        self.diffToolbar.setIconSize(qtlib.smallIconSize())
         self.diffToolbar.setStyleSheet(qtlib.tbstylesheet)
         self.diffToolbar.addAction(self.actionFind)
         hbox.addWidget(self.diffToolbar)
@@ -716,7 +718,9 @@ class DiffBrowser(QFrame):
         self._lastfile = filename
         self.clearChunks()
 
-        fd = filedata.FileData(self._ctx, None, filename, status, force=force)
+        fd = filedata.createFileData(self._ctx, None, filename, status)
+        fd.load(force=force)
+        fd.detectTextEncoding()
 
         if fd.elabel:
             self.extralabel.setText(fd.elabel)
@@ -751,7 +755,8 @@ class DiffBrowser(QFrame):
             chunk.selected = False
             chunk.write(buf)
             chunk.lines = buf.getvalue().splitlines()
-            utext += [hglib.tounicode(l) for l in chunk.lines]
+            utext += [l.decode(fd.textEncoding(), 'replace')
+                      for l in chunk.lines]
             utext.append('')
         self.sci.setText(u'\n'.join(utext))
 
