@@ -10,24 +10,24 @@ from PyQt4.QtGui import *
 
 import os
 
+from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, cmdui, resolve, thgrepo, wctxcleaner
+from tortoisehg.hgqt import qtlib, cmdcore, cmdui, resolve, thgrepo, wctxcleaner
 from tortoisehg.hgqt import csinfo, cslist
 
 BB = QDialogButtonBox
 
 class GraftDialog(QDialog):
-    showMessage = pyqtSignal(QString)
 
     def __init__(self, repoagent, parent, **opts):
         super(GraftDialog, self).__init__(parent)
         self.setWindowIcon(qtlib.geticon('hg-transplant'))
-        f = self.windowFlags()
-        self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
+        self.setWindowFlags(self.windowFlags()
+                            & ~Qt.WindowContextHelpButtonHint)
 
         self._repoagent = repoagent
+        self._cmdsession = cmdcore.nullCmdSession()
         self._graftstatefile = self.repo.join('graftstate')
-        self.aborted = False
         self.valid = True
 
         def cleanrevlist(revlist):
@@ -58,27 +58,18 @@ class GraftDialog(QDialog):
         box.setContentsMargins(*(6,)*4)
         self.setLayout(box)
 
-        destrev = self.repo['.'].rev()
-        if len(self.sourcelist) > 1:
-            listlabel = qtlib.LabeledSeparator(
-                _('Graft %d changesets on top of changeset %s') \
-                % (len(self.sourcelist), destrev))
-            self.layout().addWidget(listlabel)
-            self.cslist = cslist.ChangesetList(self.repo)
-            self.cslist.update(self.sourcelist)
-            self.layout().addWidget(self.cslist)
-
-        style = csinfo.panelstyle(selectable=True)
         self.srcb = srcb = QGroupBox()
         srcb.setLayout(QVBoxLayout())
         srcb.layout().setContentsMargins(*(2,)*4)
 
-        self.source = csinfo.create(self.repo, None, style, withupdate=True)
+        self.cslist = cslist.ChangesetList(self.repo)
         self._updateSource(0)
-        srcb.layout().addWidget(self.source)
+        srcb.layout().addWidget(self.cslist)
         self.layout().addWidget(srcb)
 
-        destb = QGroupBox( _('To graft destination'))
+        destrev = self.repo['.'].rev()
+        style = csinfo.panelstyle(selectable=True)
+        destb = QGroupBox(_('To graft destination'))
         destb.setLayout(QVBoxLayout())
         destb.layout().setContentsMargins(*(2,)*4)
         dest = csinfo.create(self.repo, destrev, style, withupdate=True)
@@ -89,36 +80,31 @@ class GraftDialog(QDialog):
         sep = qtlib.LabeledSeparator(_('Options'))
         self.layout().addWidget(sep)
 
-        self.currentuservechk = QCheckBox(_('Use my user name instead of graft '
-                                            'committer user name'))
-        self.layout().addWidget(self.currentuservechk)
+        self._optchks = {}
+        for name, text in [
+                ('currentuser', _('Use my user name instead of graft '
+                                  'committer user name')),
+                ('currentdate', _('Use current date')),
+                ('log', _('Append graft info to log message')),
+                ('autoresolve', _('Automatically resolve merge conflicts '
+                                  'where possible'))]:
+            self._optchks[name] = w = QCheckBox(text)
+            self.layout().addWidget(w)
 
-        self.currentdatevechk = QCheckBox(_('Use current date'))
-        self.layout().addWidget(self.currentdatevechk)
-
-        self.logvechk = QCheckBox(_('Append graft info to log message'))
-        self.layout().addWidget(self.logvechk)
-
-        self.autoresolvechk = QCheckBox(_('Automatically resolve merge conflicts '
-                                           'where possible'))
-        self.autoresolvechk.setChecked(
-            self.repo.ui.configbool('tortoisehg', 'autoresolve', False))
-        self.layout().addWidget(self.autoresolvechk)
-
-        self.cmd = cmdui.Widget(True, True, self)
-        self.cmd.commandFinished.connect(self.commandFinished)
-        self.showMessage.connect(self.cmd.stbar.showMessage)
-        self.cmd.stbar.linkActivated.connect(self.linkActivated)
-        self.layout().addWidget(self.cmd, 2)
+        self._cmdlog = cmdui.LogWidget(self)
+        self._cmdlog.hide()
+        self.layout().addWidget(self._cmdlog, 2)
+        self._stbar = cmdui.ThgStatusBar(self)
+        self._stbar.setSizeGripEnabled(False)
+        self._stbar.linkActivated.connect(self.linkActivated)
+        self.layout().addWidget(self._stbar)
 
         bbox = QDialogButtonBox()
         self.cancelbtn = bbox.addButton(QDialogButtonBox.Cancel)
         self.cancelbtn.clicked.connect(self.reject)
-        self.graftbtn = bbox.addButton(_('Graft'),
-                                            QDialogButtonBox.ActionRole)
+        self.graftbtn = bbox.addButton(_('Graft'), QDialogButtonBox.ActionRole)
         self.graftbtn.clicked.connect(self.graft)
-        self.abortbtn = bbox.addButton(_('Abort'),
-                                            QDialogButtonBox.ActionRole)
+        self.abortbtn = bbox.addButton(_('Abort'), QDialogButtonBox.ActionRole)
         self.abortbtn.clicked.connect(self.abort)
         self.layout().addWidget(bbox)
         self.bbox = bbox
@@ -128,7 +114,7 @@ class GraftDialog(QDialog):
         if self.checkResolve():
             self.abortbtn.setEnabled(True)
         else:
-            self.showMessage.emit(_('Checking...'))
+            self._stbar.showMessage(_('Checking...'))
             self.abortbtn.setEnabled(False)
             self.graftbtn.setEnabled(False)
             QTimer.singleShot(0, self._wctxcleaner.check)
@@ -136,12 +122,32 @@ class GraftDialog(QDialog):
         self.setMinimumWidth(480)
         self.setMaximumHeight(800)
         self.resize(0, 340)
-        self.setWindowTitle(_('Graft - %s') % self.repo.displayname)
+        self.setWindowTitle(_('Graft - %s') % repoagent.displayName())
+        self._readSettings()
 
     @property
     def repo(self):
         return self._repoagent.rawRepo()
 
+    def _readSettings(self):
+        ui = self.repo.ui
+        qs = QSettings()
+        qs.beginGroup('graft')
+        for n, w in self._optchks.iteritems():
+            if n == 'autoresolve':
+                w.setChecked(ui.configbool('tortoisehg', n,
+                                           qs.value(n, True).toBool()))
+            else:
+                w.setChecked(qs.value(n).toBool())
+        qs.endGroup()
+
+    def _writeSettings(self):
+        qs = QSettings()
+        qs.beginGroup('graft')
+        for n, w in self._optchks.iteritems():
+            qs.setValue(n, w.isChecked())
+        qs.endGroup()
+
     def _updateSourceTitle(self, idx):
         numrevs = len(self.sourcelist)
         if numrevs <= 1:
@@ -152,7 +158,7 @@ class GraftDialog(QDialog):
 
     def _updateSource(self, idx):
         self._updateSourceTitle(idx)
-        self.source.update(self.repo[self.sourcelist[idx]])
+        self.cslist.update(self.sourcelist[idx:])
 
     @pyqtSlot(bool)
     def _onCheckFinished(self, clean):
@@ -164,37 +170,31 @@ class GraftDialog(QDialog):
         else:
             self.graftbtn.setEnabled(True)
             txt = _('You may continue or start the graft')
-        self.showMessage.emit(txt)
+        self._stbar.showMessage(txt)
 
     def graft(self):
         self.graftbtn.setEnabled(False)
         self.cancelbtn.setShown(False)
-        cmdline = ['graft', '--repository', self.repo.root]
-        cmdline += ['--config', 'ui.merge=internal:' +
-                    (self.autoresolvechk.isChecked() and 'merge' or 'fail')]
-        if self.currentuservechk.isChecked():
-            cmdline += ['--currentuser']
-        if self.currentdatevechk.isChecked():
-            cmdline += ['--currentdate']
-        if self.logvechk.isChecked():
-            cmdline += ['--log']
+        opts = dict((n, w.isChecked()) for n, w in self._optchks.iteritems())
+        itool = opts.pop('autoresolve') and 'merge' or 'fail'
+        opts['config'] = 'ui.merge=internal:%s' % itool
         if os.path.exists(self._graftstatefile):
-            cmdline += ['--continue']
+            opts['continue'] = True
+            args = []
         else:
-            for source in self.sourcelist:
-                cmdline += [str(source)]
-        self.repo.incrementBusyCount()
-        self.cmd.run(cmdline)
+            args = [hglib.tounicode(str(s)) for s in self.sourcelist]
+        cmdline = hglib.buildcmdargs('graft', *args, **opts)
+        sess = self._runCommand(cmdline)
+        sess.commandFinished.connect(self._graftFinished)
 
     def abort(self):
         self.abortbtn.setDisabled(True)
         if os.path.exists(self._graftstatefile):
             # Remove the existing graftstate file!
             os.remove(self._graftstatefile)
-        cmdline = ['update', '--repository', self.repo.root, '--clean', '--rev', 'p1()']
-        self.repo.incrementBusyCount()
-        self.aborted = True
-        self.cmd.run(cmdline)
+        cmdline = hglib.buildcmdargs('update', clean=True, rev='p1()')
+        sess = self._runCommand(cmdline)
+        sess.commandFinished.connect(self._abortFinished)
 
     def graftstate(self):
         graftstatefile = self.repo.join('graftstate')
@@ -209,22 +209,38 @@ class GraftDialog(QDialog):
                     return revlist
         return None
 
-    def commandFinished(self, ret):
-        self.repo.decrementBusyCount()
-        if  self.aborted or self.checkResolve() is False:
+    def _runCommand(self, cmdline):
+        assert self._cmdsession.isFinished()
+        self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
+        sess.commandFinished.connect(self._stbar.clearProgress)
+        sess.outputReceived.connect(self._cmdlog.appendLog)
+        sess.progressReceived.connect(self._stbar.setProgress)
+        cmdui.updateStatusMessage(self._stbar, sess)
+        return sess
+
+    @pyqtSlot(int)
+    def _graftFinished(self, ret):
+        if self.checkResolve() is False:
             msg = _('Graft is complete')
-            if self.aborted:
-                msg = _('Graft aborted')
-            elif ret == 255:
+            if ret == 255:
                 msg = _('Graft failed')
-                self.cmd.setShowOutput(True)  # contains hint
+                self._cmdlog.show()  # contains hint
             else:
                 self._updateSource(len(self.sourcelist) - 1)
-            self.showMessage.emit(msg)
-            self.graftbtn.setEnabled(True)
-            self.graftbtn.setText(_('Close'))
-            self.graftbtn.clicked.disconnect(self.graft)
-            self.graftbtn.clicked.connect(self.accept)
+            self._stbar.showMessage(msg)
+            self._makeCloseButton()
+
+    @pyqtSlot()
+    def _abortFinished(self):
+        if self.checkResolve() is False:
+            self._stbar.showMessage(_('Graft aborted'))
+            self._makeCloseButton()
+
+    def _makeCloseButton(self):
+        self.graftbtn.setEnabled(True)
+        self.graftbtn.setText(_('Close'))
+        self.graftbtn.clicked.disconnect(self.graft)
+        self.graftbtn.clicked.connect(self.accept)
 
     def checkResolve(self):
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
@@ -236,7 +252,7 @@ class GraftDialog(QDialog):
         else:
             self.graftbtn.setEnabled(True)
             txt = _('You may continue the graft')
-        self.showMessage.emit(txt)
+        self._stbar.showMessage(txt)
 
         currgraftrevs = self.graftstate()
         if currgraftrevs:
@@ -277,3 +293,7 @@ class GraftDialog(QDialog):
                                         labels=labels, parent=self):
                 return
         super(GraftDialog, self).reject()
+
+    def done(self, r):
+        self._writeSettings()
+        super(GraftDialog, self).done(r)
