@@ -9,7 +9,7 @@ import os
 
 from mercurial import hg, ui, mdiff, similar, patch
 
-from tortoisehg.util import hglib, shlib, thread2
+from tortoisehg.util import hglib, thread2
 
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, htmlui, cmdui
@@ -29,11 +29,11 @@ class DetectRenameDialog(QDialog):
         QDialog.__init__(self, parent)
 
         self._repoagent = repoagent
-        repo = repoagent.rawRepo()
         self.pats = pats
         self.thread = None
 
-        self.setWindowTitle(_('Detect Copies/Renames in %s') % repo.displayname)
+        self.setWindowTitle(_('Detect Copies/Renames in %s')
+                            % repoagent.displayName())
         self.setWindowIcon(qtlib.geticon('detect_rename'))
         self.setWindowFlags(Qt.Window)
 
@@ -166,7 +166,7 @@ class DetectRenameDialog(QDialog):
         self.repo.thginvalidate()
         self.repo.lfstatus = True
         wctx = self.repo[None]
-        wctx.status(unknown=True)
+        hglib.querywctxstatus(wctx, unknown=True)
         self.repo.lfstatus = False
         self.unrevlist.clear()
         dests = []
@@ -219,7 +219,7 @@ class DetectRenameDialog(QDialog):
         self.thread.start()
 
     def searchfinished(self):
-        self.stbar.clear()
+        self.stbar.clearProgress()
         for col in xrange(3):
             self.matchtv.resizeColumnToContents(col)
         self.findbtn.setEnabled(self.unrevlist.count())
@@ -436,7 +436,7 @@ class RenameSearchThread(QThread):
         wctx = repo[None]
         pctx = repo['.']
         if self.copies:
-            wctx.status(clean=True)
+            hglib.querywctxstatus(wctx, clean=True)
             srcs = wctx.removed() + wctx.deleted()
             srcs += wctx.modified() + wctx.clean()
         else:
