@@ -11,21 +11,20 @@ from mercurial import match
 from tortoisehg.hgqt import qtlib
 from tortoisehg.hgqt.i18n import _
 
-class LfilesPrompt(qtlib.CustomPrompt):
-    def __init__(self, parent, files=None):
-        qtlib.CustomPrompt.__init__(self, _('Confirm Add'),
-                                    _('Some of the files that you have selected are of a size '
-                                      'over 10 MB.  You may make more efficient use of disk space '
-                                      'by adding these files as largefiles, which will store only the '
-                                      'most recent revision of each file in your local repository, '
-                                      'with older revisions available on the server.  Do you wish '
-                                      'to add these files as largefiles?'), parent,
-                                      (_('Add as &Largefiles'), _('Add as &Normal Files'), _('Cancel')),
-                                      0, 2, files)
+def _createPrompt(parent, files):
+    return qtlib.CustomPrompt(
+        _('Confirm Add'),
+        _('Some of the files that you have selected are of a size '
+          'over 10 MB.  You may make more efficient use of disk space '
+          'by adding these files as largefiles, which will store only the '
+          'most recent revision of each file in your local repository, '
+          'with older revisions available on the server.  Do you wish '
+          'to add these files as largefiles?'), parent,
+        (_('Add as &Largefiles'), _('Add as &Normal Files'), _('Cancel')),
+        0, 2, files)
 
 def promptForLfiles(parent, ui, repo, files):
     lfiles = []
-    uself = 'largefiles' in repo.extensions()
     section = 'largefiles'
     try:
         minsize = int(ui.config(section, 'minsize', default='10'))
@@ -47,7 +46,7 @@ def promptForLfiles(parent, ui, repo, files):
             if filesize > minsize*1024*1024:
                 lfiles.append(wfile)
     if lfiles:
-        ret = LfilesPrompt(parent, files).run()
+        ret = _createPrompt(parent, files).run()
         if ret == 0:
             # add as largefiles/bfiles
             for lfile in lfiles:
