@@ -10,7 +10,7 @@ from mercurial import error, util
 
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, csinfo, i18n, cmdui, status, resolve
+from tortoisehg.hgqt import qtlib, csinfo, cmdcore, cmdui, status, resolve
 from tortoisehg.hgqt import qscilib, thgrepo, messageentry, commit, wctxcleaner
 
 from PyQt4.QtCore import *
@@ -22,23 +22,19 @@ class MergeDialog(QWizard):
 
     def __init__(self, repoagent, otherrev, parent=None):
         super(MergeDialog, self).__init__(parent)
+        self._repoagent = repoagent
         f = self.windowFlags()
         self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
-
-        repo = repoagent.rawRepo()
-        self.otherrev = str(otherrev)
-        self.localrev = str(repo['.'].rev())
-
-        self.setWindowTitle(_('Merge - %s') % repo.displayname)
+        self.setWindowTitle(_('Merge - %s') % repoagent.displayName())
         self.setWindowIcon(qtlib.geticon('hg-merge'))
         self.setOption(QWizard.NoBackButtonOnStartPage, True)
         self.setOption(QWizard.NoBackButtonOnLastPage, True)
         self.setOption(QWizard.IndependentPages, True)
 
         # set pages
-        summarypage = SummaryPage(repoagent, self)
+        summarypage = SummaryPage(repoagent, str(otherrev), self)
         self.addPage(summarypage)
-        self.addPage(MergePage(repoagent, self))
+        self.addPage(MergePage(repoagent, str(otherrev), self))
         self.addPage(CommitPage(repoagent, self))
         self.addPage(ResultPage(repoagent, self))
         self.currentIdChanged.connect(self.pageChanged)
@@ -52,6 +48,26 @@ class MergeDialog(QWizard):
         repoagent.repositoryChanged.connect(self.repositoryChanged)
         repoagent.configChanged.connect(self.configChanged)
 
+        self._readSettings()
+
+    def _readSettings(self):
+        qs = QSettings()
+        qs.beginGroup('merge')
+        for n in ['autoadvance', 'skiplast']:
+            self.setField(n, qs.value(n, False))
+        repo = self._repoagent.rawRepo()
+        n = 'autoresolve'
+        self.setField(n, repo.ui.configbool('tortoisehg', n,
+                                            qs.value(n, True).toBool()))
+        qs.endGroup()
+
+    def _writeSettings(self):
+        qs = QSettings()
+        qs.beginGroup('merge')
+        for n in ['autoadvance', 'autoresolve', 'skiplast']:
+            qs.setValue(n, self.field(n))
+        qs.endGroup()
+
     @pyqtSlot()
     def repositoryChanged(self):
         if self.currentPage():
@@ -70,6 +86,10 @@ class MergeDialog(QWizard):
         if self.currentPage().canExit():
             super(MergeDialog, self).reject()
 
+    def done(self, r):
+        self._writeSettings()
+        super(MergeDialog, self).done(r)
+
 
 class BasePage(QWizardPage):
     def __init__(self, repoagent, parent):
@@ -116,17 +136,12 @@ class BasePage(QWizardPage):
 class SummaryPage(BasePage):
     refreshFinished = pyqtSignal()
 
-    def __init__(self, repoagent, parent):
+    def __init__(self, repoagent, otherrev, parent):
         super(SummaryPage, self).__init__(repoagent, parent)
         self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
         self._wctxcleaner.checkStarted.connect(self._onCheckStarted)
         self._wctxcleaner.checkFinished.connect(self._onCheckFinished)
 
-    ### Override Methods ###
-
-    def initializePage(self):
-        if self.layout():
-            return
         self.setTitle(_('Prepare to merge'))
         self.setSubTitle(_('Verify merge targets and ensure your working '
                            'directory is clean.'))
@@ -146,19 +161,14 @@ class SummaryPage(BasePage):
         ## merge target
         other_sep = qtlib.LabeledSeparator(_('Merge from (other revision)'))
         self.layout().addWidget(other_sep)
-        try:
-            otherCsInfo = create(self.wizard().otherrev)
-            self.layout().addWidget(otherCsInfo)
-            self.otherCsInfo = otherCsInfo
-        except error.RepoLookupError:
-            qtlib.InfoMsgBox(_('Unable to merge'),
-                             _('Merge revision not specified or not found'))
-            QTimer.singleShot(0, self.wizard().close)
+        otherCsInfo = create(otherrev)
+        self.layout().addWidget(otherCsInfo)
+        self.otherCsInfo = otherCsInfo
 
         ## current revision
         local_sep = qtlib.LabeledSeparator(_('Merge to (working directory)'))
         self.layout().addWidget(local_sep)
-        localCsInfo = create(self.wizard().localrev)
+        localCsInfo = create(str(repo['.'].rev()))
         self.layout().addWidget(localCsInfo)
         self.localCsInfo = localCsInfo
 
@@ -210,31 +220,20 @@ class SummaryPage(BasePage):
         self.groups.add(force_chk, 'dirty')
         wdbox.addWidget(force_chk)
 
-        ### options
-        expander = qtlib.ExpanderLabel(_('Options'), False)
-        expander.expanded.connect(self.toggleShowOptions)
-        self.layout().addWidget(expander)
-        self.expander = expander
-
         ### discard option
         discard_chk = QCheckBox(_('Discard all changes from merge target '
                                   '(other) revision'))
         self.registerField('discard', discard_chk)
         self.layout().addWidget(discard_chk)
-        self.discard_chk = discard_chk
 
         ## auto-resolve
         autoresolve_chk = QCheckBox(_('Automatically resolve merge conflicts '
                                       'where possible'))
-        autoresolve_chk.setChecked(
-            repo.ui.configbool('tortoisehg', 'autoresolve', False))
         self.registerField('autoresolve', autoresolve_chk)
         self.layout().addWidget(autoresolve_chk)
-        self.autoresolve_chk = autoresolve_chk
 
         self.groups.set_visible(False, 'dirty')
         self.groups.set_visible(False, 'merged')
-        self.toggleShowOptions(self.expander.is_expanded())
 
     def isComplete(self):
         'should Next button be sensitive?'
@@ -252,19 +251,14 @@ class SummaryPage(BasePage):
                       % (self.otherCsInfo.get_data('revid')),
                          labels=labels, parent=self):
                 return False
-        return super(SummaryPage, self).validatePage();
+        return super(SummaryPage, self).validatePage()
 
     ## custom methods ##
 
-    def toggleShowOptions(self, visible):
-        self.discard_chk.setShown(visible)
-        self.autoresolve_chk.setShown(visible)
-
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
         pctx = self.repo['.']
         self.localCsInfo.update(pctx)
-        self.wizard().localrev = str(pctx.rev())
 
     def canExit(self):
         'can merge tool be closed?'
@@ -309,50 +303,49 @@ class SummaryPage(BasePage):
 
 
 class MergePage(BasePage):
-    def __init__(self, repoagent, parent):
+    def __init__(self, repoagent, otherrev, parent):
         super(MergePage, self).__init__(repoagent, parent)
+        self._otherrev = otherrev
         self.mergecomplete = False
 
         self.setTitle(_('Merging...'))
         self.setSubTitle(_('All conflicting files will be marked unresolved.'))
         self.setLayout(QVBoxLayout())
 
-        self.cmd = cmdui.Widget(True, False, self)
-        self.cmd.commandFinished.connect(self.onCommandFinished)
-        self.cmd.setShowOutput(True)
-        self.layout().addWidget(self.cmd)
+        self._cmdsession = cmdcore.nullCmdSession()
+        self._cmdlog = cmdui.LogWidget(self)
+        self.layout().addWidget(self._cmdlog)
 
         self.reslabel = QLabel()
         self.reslabel.linkActivated.connect(self.onLinkActivated)
         self.reslabel.setWordWrap(True)
         self.layout().addWidget(self.reslabel)
 
-        self.autonext = QCheckBox(_('Automatically advance to next page '
-                                    'when merge is complete.'))
-        checked = QSettings().value('merge/autoadvance', False).toBool()
-        self.autonext.setChecked(checked)
-        self.autonext.toggled.connect(self.tryAutoAdvance)
-        self.layout().addWidget(self.autonext)
+        autonext = QCheckBox(_('Automatically advance to next page '
+                               'when merge is complete.'))
+        autonext.clicked.connect(self.tryAutoAdvance)
+        self.registerField('autoadvance', autonext)
+        self.layout().addWidget(autonext)
 
     def currentPage(self):
         super(MergePage, self).currentPage()
         if self.field('discard').toBool():
             # '.' is safer than self.localrev, in case the user has
             # pulled a fast one on us and updated from the CLI
-            cmdline = ['--repository', self.repo.root, 'debugsetparents',
-                       '.', self.wizard().otherrev]
+            cmdline = ['debugsetparents', '.', hglib.tounicode(self._otherrev)]
         else:
-            cmdline = ['--repository', self.repo.root, 'merge', '--verbose']
+            cmdline = ['merge', '--verbose']
             if self.field('force').toBool():
                 cmdline.append('--force')
             tool = self.field('autoresolve').toBool() and 'merge' or 'fail'
             cmdline += ['--tool=internal:' + tool]
-            cmdline.append(self.wizard().otherrev)
+            cmdline.append(hglib.tounicode(self._otherrev))
 
         if len(self.repo.parents()) == 1:
-            self.repo.incrementBusyCount()
-            self.cmd.core.clearOutput()
-            self.cmd.run(cmdline)
+            self._cmdlog.clearLog()
+            self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
+            sess.commandFinished.connect(self.onCommandFinished)
+            sess.outputReceived.connect(self._cmdlog.appendLog)
         else:
             self.mergecomplete = True
             self.completeChanged.emit()
@@ -361,38 +354,43 @@ class MergePage(BasePage):
         'should Next button be sensitive?'
         if not self.mergecomplete:
             return False
-        count = 0
+        ucount = 0
+        rcount = 0
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
             if status == 'u':
-                count += 1
-        if count:
+                ucount += 1
+            if status == 'r':
+                rcount += 1
+        if ucount:
             if self.field('autoresolve').toBool():
                 # if autoresolve is enabled, we know these were real conflicts
                 self.reslabel.setText(_('%d files have <b>merge conflicts</b> '
                                         'that must be <a href="resolve">'
-                                        '<b>resolved</b></a>') % count)
+                                        '<b>resolved</b></a>') % ucount)
             else:
                 # else give a calmer indication of conflicts
                 self.reslabel.setText(_('%d files were modified on both '
                                         'branches and must be <a href="resolve">'
-                                        '<b>resolved</b></a>') % count)
+                                        '<b>resolved</b></a>') % ucount)
             return False
+        elif rcount:
+            self.reslabel.setText(_('No merge conflicts, ready to commit or '
+                                    '<a href="resolve"><b>review</b></a>'))
         else:
             self.reslabel.setText(_('No merge conflicts, ready to commit'))
         return True
 
+    @pyqtSlot(bool)
     def tryAutoAdvance(self, checked):
         if checked and self.isComplete():
             self.wizard().next()
 
-    def cleanupPage(self):
-        QSettings().setValue('merge/autoadvance', self.autonext.isChecked())
-
+    @pyqtSlot(int)
     def onCommandFinished(self, ret):
-        self.repo.decrementBusyCount()
+        sess = self._cmdsession
         if ret in (0, 1):
             self.mergecomplete = True
-            if self.autonext.isChecked():
+            if self.field('autoadvance').toBool() and not sess.warningString():
                 self.tryAutoAdvance(True)
             self.completeChanged.emit()
 
@@ -400,9 +398,8 @@ class MergePage(BasePage):
     def onLinkActivated(self, cmd):
         if cmd == 'resolve':
             dlg = resolve.ResolveDialog(self._repoagent, self)
-            dlg.finished.connect(dlg.deleteLater)
             dlg.exec_()
-            if self.autonext.isChecked():
+            if self.field('autoadvance').toBool():
                 self.tryAutoAdvance(True)
             self.completeChanged.emit()
 
@@ -485,10 +482,10 @@ class CommitPage(BasePage):
         self.layout().addWidget(msgEntry)
         self.msgEntry = msgEntry
 
-        self.cmd = cmdui.Widget(True, False, self)
-        self.cmd.commandFinished.connect(self.onCommandFinished)
-        self.cmd.setShowOutput(False)
-        self.layout().addWidget(self.cmd)
+        self._cmdsession = cmdcore.nullCmdSession()
+        self._cmdlog = cmdui.LogWidget(self)
+        self._cmdlog.hide()
+        self.layout().addWidget(self._cmdlog)
 
         self.delayednext = False
 
@@ -500,11 +497,10 @@ class CommitPage(BasePage):
         actionEnter.triggered.connect(tryperform)
         self.addAction(actionEnter)
 
-        self.skiplast = QCheckBox(_('Skip final confirmation page, '
-                                    'close after commit.'))
-        checked = QSettings().value('merge/skiplast', False).toBool()
-        self.skiplast.setChecked(checked)
-        self.layout().addWidget(self.skiplast)
+        skiplast = QCheckBox(_('Skip final confirmation page, '
+                               'close after commit.'))
+        self.registerField('skiplast', skiplast)
+        self.layout().addWidget(skiplast)
 
         hblayout = QHBoxLayout()
         self.opts = commit.readopts(self.repo.ui)
@@ -532,7 +528,6 @@ class CommitPage(BasePage):
 
     def cleanupPage(self):
         s = QSettings()
-        s.setValue('merge/skiplast', self.skiplast.isChecked())
         self.msgEntry.saveSettings(s, 'merge/message')
 
     def currentPage(self):
@@ -540,17 +535,7 @@ class CommitPage(BasePage):
         self.wizard().setOption(QWizard.NoDefaultButton, True)
         self.mergeCsInfo.update()  # show post-merge state
 
-        engmsg = self.repo.ui.configbool('tortoisehg', 'engmsg', False)
-        wctx = self.repo[None]
-        if wctx.p1().branch() == wctx.p2().branch():
-            msgset = i18n.keepgettext()._('Merge')
-            text = engmsg and msgset['id'] or msgset['str']
-            text = unicode(text)
-        else:
-            msgset = i18n.keepgettext()._('Merge with %s')
-            text = engmsg and msgset['id'] or msgset['str']
-            text = unicode(text) % hglib.tounicode(wctx.p2().branch())
-        self.msgEntry.setText(text)
+        self.msgEntry.setText(commit.mergecommitmessage(self.repo))
         self.msgEntry.moveCursorToEnd()
 
     @pyqtSlot(QString)
@@ -564,33 +549,32 @@ class CommitPage(BasePage):
         return len(self.repo.parents()) == 2 and len(self.msgEntry.text()) > 0
 
     def validatePage(self):
-
-        if self.cmd.core.running():
+        if not self._cmdsession.isFinished():
             return False
 
         if len(self.repo.parents()) == 1:
             # commit succeeded, repositoryChanged() called wizard().next()
-            if self.skiplast.isChecked():
+            if self.field('skiplast').toBool():
                 self.wizard().close()
             return True
 
-        user = qtlib.getCurrentUsername(self, self.repo, self.opts)
+        user = hglib.tounicode(qtlib.getCurrentUsername(self, self.repo,
+                                                        self.opts))
         if not user:
             return False
 
         self.setTitle(_('Committing...'))
         self.setSubTitle(_('Please wait while committing merged files.'))
 
-        message = hglib.fromunicode(self.msgEntry.text())
-        cmdline = ['commit', '--verbose', '--message', message,
-                   '--repository', self.repo.root, '--user', user]
+        message = unicode(self.msgEntry.text())
+        cmdline = ['commit', '--verbose', '--message', message, '--user', user]
         if self.opts.get('recurseinsubrepos'):
             cmdline.append('--subrepos')
         try:
             date = self.opts.get('date')
             if date:
                 util.parsedate(date)
-                dcmd = ['--date', date]
+                dcmd = ['--date', hglib.tounicode(date)]
             else:
                 dcmd = []
         except error.Abort, e:
@@ -609,24 +593,26 @@ class CommitPage(BasePage):
         commandlines = [cmdline]
         pushafter = self.repo.ui.config('tortoisehg', 'cipushafter')
         if pushafter:
-            cmd = ['push', '--repository', self.repo.root, pushafter]
+            cmd = ['push', hglib.tounicode(pushafter)]
             commandlines.append(cmd)
-        self.repo.incrementBusyCount()
-        self.cmd.setShowOutput(True)
-        self.cmd.run(*commandlines)
+        self._cmdlog.show()
+        sess = self._repoagent.runCommandSequence(commandlines, self)
+        self._cmdsession = sess
+        sess.commandFinished.connect(self.onCommandFinished)
+        sess.outputReceived.connect(self._cmdlog.appendLog)
         return False
 
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
         if len(self.repo.parents()) == 1:
-            if self.cmd.core.running():
+            if not self._cmdsession.isFinished():
                 # call self.wizard().next() after the current command finishes
                 self.delayednext = True
             else:
                 self.wizard().next()
 
-    def onCommandFinished(self, ret):
-        self.repo.decrementBusyCount()
+    @pyqtSlot()
+    def onCommandFinished(self):
         if self.delayednext:
             self.delayednext = False
             self.wizard().next()
@@ -641,7 +627,8 @@ class CommitPage(BasePage):
 
     def details(self):
         self.userhist = self.readUserHistory()
-        dlg = commit.DetailsDialog(self.opts, self.userhist, self, mode='merge')
+        dlg = commit.DetailsDialog(self._repoagent, self.opts, self.userhist,
+                                   self, mode='merge')
         dlg.finished.connect(dlg.deleteLater)
         dlg.setWindowFlags(Qt.Sheet)
         dlg.setWindowModality(Qt.WindowModal)
