@@ -120,40 +120,41 @@ class _SciImSupport(object):
 class ScintillaCompat(QsciScintilla):
     """Scintilla widget with compatibility patches"""
 
-    def __init__(self, parent=None):
-        super(ScintillaCompat, self).__init__(parent)
-        self._imsupport = _SciImSupport(self)
+    # QScintilla 2.8 can handle input method events properly
+    if QSCINTILLA_VERSION < 0x20800:
+        def __init__(self, parent=None):
+            super(ScintillaCompat, self).__init__(parent)
+            self._imsupport = _SciImSupport(self)
 
-    def inputMethodQuery(self, query):
-        if query == Qt.ImMicroFocus:
-            return self._cursorRect()
-        return super(ScintillaCompat, self).inputMethodQuery(query)
+        def inputMethodQuery(self, query):
+            if query == Qt.ImMicroFocus:
+                # a rectangle (in viewport coords) including the cursor
+                l, i = self.getCursorPosition()
+                p = self.positionFromLineIndex(l, i)
+                x = self.SendScintilla(QsciScintilla.SCI_POINTXFROMPOSITION,
+                                       0, p)
+                y = self.SendScintilla(QsciScintilla.SCI_POINTYFROMPOSITION,
+                                       0, p)
+                w = self.SendScintilla(QsciScintilla.SCI_GETCARETWIDTH)
+                return QRect(x, y, w, self.textHeight(l))
+            return super(ScintillaCompat, self).inputMethodQuery(query)
 
-    def inputMethodEvent(self, event):
-        if self.isReadOnly():
-            return
+        def inputMethodEvent(self, event):
+            if self.isReadOnly():
+                return
 
-        self.removeSelectedText()
-        self._imsupport.removepreedit()
-        self._imsupport.commitstr(event.replacementStart(),
-                                  event.replacementLength(),
-                                  event.commitString())
-        self._imsupport.insertpreedit(event.preeditString())
-        for a in event.attributes():
-            if a.type == QInputMethodEvent.Cursor:
-                self._imsupport.movepreeditcursor(a.start)
-            # TODO TextFormat
+            self.removeSelectedText()
+            self._imsupport.removepreedit()
+            self._imsupport.commitstr(event.replacementStart(),
+                                      event.replacementLength(),
+                                      event.commitString())
+            self._imsupport.insertpreedit(event.preeditString())
+            for a in event.attributes():
+                if a.type == QInputMethodEvent.Cursor:
+                    self._imsupport.movepreeditcursor(a.start)
+                # TextFormat is not supported
 
-        event.accept()
-
-    def _cursorRect(self):
-        """Return a rectangle (in viewport coords) including the cursor"""
-        l, i = self.getCursorPosition()
-        p = self.positionFromLineIndex(l, i)
-        x = self.SendScintilla(QsciScintilla.SCI_POINTXFROMPOSITION, 0, p)
-        y = self.SendScintilla(QsciScintilla.SCI_POINTYFROMPOSITION, 0, p)
-        w = self.SendScintilla(QsciScintilla.SCI_GETCARETWIDTH)
-        return QRect(x, y, w, self.textHeight(l))
+            event.accept()
 
     # QScintilla 2.5 can translate Backtab to Shift+SCK_TAB (issue #82)
     if QSCINTILLA_VERSION < 0x20500:
@@ -264,11 +265,6 @@ class Scintilla(ScintillaCompat):
             self.SendScintilla(QsciScintilla.SCI_SETVIRTUALSPACEOPTIONS,
                                QsciScintilla.SCVS_RECTANGULARSELECTION)
 
-    def read(self, f):
-        result = super(Scintilla, self).read(f)
-        self.setDefaultEolMode()
-        return result
-
     def contextMenuEvent(self, event):
         menu = self.createEditorContextMenu()
         menu.exec_(event.globalPos())
@@ -499,16 +495,16 @@ class SearchToolBar(QToolBar):
     searchRequested = pyqtSignal(unicode, bool, bool, bool)
     """Emitted (pattern, icase, wrap, forward) when requested"""
 
-    def __init__(self, parent=None, hidable=False):
+    def __init__(self, parent=None):
         super(SearchToolBar, self).__init__(_('Search'), parent,
-                                            objectName='search',
-                                            iconSize=QSize(16, 16))
-        if hidable:
-            self._close_button = QToolButton(icon=qtlib.geticon('window-close'),
-                                             shortcut=Qt.Key_Escape)
-            self._close_button.clicked.connect(self.hide)
-            self.addWidget(self._close_button)
-            self.addWidget(qtlib.Spacer(2, 2))
+                                            objectName='search')
+        self.setIconSize(qtlib.smallIconSize())
+
+        a = self.addAction(qtlib.geticon('window-close'), '')
+        a.setShortcut(Qt.Key_Escape)
+        a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
+        a.triggered.connect(self.hide)
+        self.addWidget(qtlib.Spacer(2, 2))
 
         self._le = QLineEdit()
         if hasattr(self._le, 'setPlaceholderText'): # Qt >= 4.7
@@ -525,16 +521,19 @@ class SearchToolBar(QToolBar):
         self.addWidget(self._chk)
         self._wrapchk = QCheckBox(_('Wrap search'))
         self.addWidget(self._wrapchk)
-        self._btprev = QPushButton(_('Prev'), icon=qtlib.geticon('go-up'),
-                                   iconSize=QSize(16, 16))
-        self._btprev.clicked.connect(
-            lambda: self._emitSearchRequested(forward=False))
-        self.addWidget(self._btprev)
-        self._bt = QPushButton(_('Next'), icon=qtlib.geticon('go-down'),
-                               iconSize=QSize(16, 16))
-        self._bt.clicked.connect(self._emitSearchRequested)
+
+        self._prevact = self.addAction(qtlib.geticon('go-up'), _('Prev'))
+        self._prevact.setShortcuts(QKeySequence.FindPrevious)
+        self._nextact = self.addAction(qtlib.geticon('go-down'), _('Next'))
+        self._nextact.setShortcuts(QKeySequence.FindNext)
+        for a in [self._prevact, self._nextact]:
+            a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
+            a.triggered.connect(self._emitSearchRequested)
+            w = self.widgetForAction(a)
+            w.setAutoRaise(False)  # no flat button
+            w.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+
         self._le.textChanged.connect(self._updateSearchButtons)
-        self.addWidget(self._bt)
 
         self.setFocusProxy(self._le)
         self.setStyleSheet(qtlib.tbstylesheet)
@@ -551,22 +550,16 @@ class SearchToolBar(QToolBar):
         self._updateSearchButtons()
 
     def keyPressEvent(self, event):
-        if event.matches(QKeySequence.FindNext):
-            self._emitSearchRequested(forward=True)
-            return
-        if event.matches(QKeySequence.FindPrevious):
-            self._emitSearchRequested(forward=False)
-            return
         if event.key() in (Qt.Key_Enter, Qt.Key_Return):
             return  # handled by returnPressed
         super(SearchToolBar, self).keyPressEvent(event)
 
     def wheelEvent(self, event):
         if event.delta() > 0:
-            self._emitSearchRequested(forward=False)
+            self._prevact.trigger()
             return
         if event.delta() < 0:
-            self._emitSearchRequested(forward=True)
+            self._nextact.trigger()
             return
         super(SearchToolBar, self).wheelEvent(event)
 
@@ -590,16 +583,21 @@ class SearchToolBar(QToolBar):
         self.conditionChanged.emit(self.pattern(), self.caseInsensitive(),
                                    self.wrapAround())
 
-    @pyqtSlot()
-    def _emitSearchRequested(self, forward=True):
+    @qtlib.senderSafeSlot()
+    def _emitSearchRequested(self):
+        forward = self.sender() is not self._prevact
         self.searchRequested.emit(self.pattern(), self.caseInsensitive(),
                                   self.wrapAround(), forward)
 
+    def editorActions(self):
+        """List of actions that should be available in main editor widget"""
+        return [self._prevact, self._nextact]
+
     @pyqtSlot()
     def _updateSearchButtons(self):
         enabled = bool(self._le.text())
-        self._btprev.setEnabled(enabled)
-        self._bt.setEnabled(enabled)
+        for a in [self._prevact, self._nextact]:
+            a.setEnabled(enabled)
 
     def pattern(self):
         """Returns the current search pattern [unicode]"""
@@ -695,6 +693,79 @@ def findTabIndentsInLines(lines, linestocheck=100):
             return True
     return False # Use spaces for indents default
 
+def readFile(editor, filename, encoding=None):
+    f = QFile(filename)
+    if not f.open(QIODevice.ReadOnly):
+        qtlib.WarningMsgBox(_('Unable to read file'),
+                            _('Could not open the specified file for reading.'),
+                            f.errorString(), parent=editor)
+        return False
+    try:
+        earlybytes = f.read(4096)
+        if '\0' in earlybytes:
+            qtlib.WarningMsgBox(_('Unable to read file'),
+                                _('This appears to be a binary file.'),
+                                parent=editor)
+            return False
+
+        f.seek(0)
+        data = str(f.readAll())
+        if f.error():
+            qtlib.WarningMsgBox(_('Unable to read file'),
+                                _('An error occurred while reading the file.'),
+                                f.errorString(), parent=editor)
+            return False
+    finally:
+        f.close()
+
+    if encoding:
+        try:
+            text = data.decode(encoding)
+        except UnicodeDecodeError, inst:
+            qtlib.WarningMsgBox(_('Text Translation Failure'),
+                                _('Could not translate the file content from '
+                                  'native encoding.'),
+                                (_('Several characters would be lost.')
+                                 + '\n\n' + hglib.tounicode(str(inst))),
+                                parent=editor)
+            text = data.decode(encoding, 'replace')
+    else:
+        text = hglib.tounicode(data)
+    editor.setText(text)
+    editor.setDefaultEolMode()
+    editor.setModified(False)
+    return True
+
+def writeFile(editor, filename, encoding=None):
+    text = editor.text()
+    try:
+        if encoding:
+            data = unicode(text).encode(encoding)
+        else:
+            data = hglib.fromunicode(text)
+    except UnicodeEncodeError, inst:
+        qtlib.WarningMsgBox(_('Unable to write file'),
+                            _('Could not translate the file content to '
+                              'native encoding.'),
+                            hglib.tounicode(str(inst)), parent=editor)
+        return False
+
+    f = QFile(filename)
+    if not f.open(QIODevice.WriteOnly):
+        qtlib.WarningMsgBox(_('Unable to write file'),
+                            _('Could not open the specified file for writing.'),
+                            f.errorString(), parent=editor)
+        return False
+    try:
+        if f.write(data) < 0:
+            qtlib.WarningMsgBox(_('Unable to write file'),
+                                _('An error occurred while writing the file.'),
+                                f.errorString(), parent=editor)
+            return False
+    finally:
+        f.close()
+    return True
+
 def fileEditor(filename, **opts):
     'Open a simple modal file editing dialog'
     dialog = QDialog()
@@ -713,7 +784,7 @@ def fileEditor(filename, **opts):
         editor.setFolding(QsciScintilla.BoxedTreeFoldStyle)
     dialog.layout().addWidget(editor)
 
-    searchbar = SearchToolBar(dialog, hidable=True)
+    searchbar = SearchToolBar(dialog)
     searchbar.searchRequested.connect(editor.find)
     searchbar.conditionChanged.connect(editor.highlightText)
     searchbar.hide()
@@ -724,6 +795,7 @@ def fileEditor(filename, **opts):
         searchbar.show()
         searchbar.setFocus(Qt.OtherFocusReason)
     qtlib.newshortcutsforstdkey(QKeySequence.Find, dialog, showsearchbar)
+    dialog.addActions(searchbar.editorActions())
     dialog.layout().addWidget(searchbar)
 
     BB = QDialogButtonBox
@@ -738,20 +810,12 @@ def fileEditor(filename, **opts):
     dialog.resize(desktopgeom.size() * 0.5)
     dialog.restoreGeometry(s.value(geomname).toByteArray())
 
-    ret = QDialog.Rejected
-    try:
-        f = QFile(filename)
-        f.open(QIODevice.ReadOnly)
-        editor.read(f)
-        editor.setModified(False)
-
-        ret = dialog.exec_()
-        if ret == QDialog.Accepted:
-            f = QFile(filename)
-            f.open(QIODevice.WriteOnly)
-            editor.write(f)
-        s.setValue(geomname, dialog.saveGeometry())
-    except EnvironmentError, e:
-        qtlib.WarningMsgBox(_('Unable to read/write config file'),
-                            hglib.tounicode(str(e)), parent=dialog)
+    if not readFile(editor, filename):
+        return QDialog.Rejected
+    ret = dialog.exec_()
+    if ret != QDialog.Accepted:
+        return ret
+    if not writeFile(editor, filename):
+        return QDialog.Rejected
+    s.setValue(geomname, dialog.saveGeometry())
     return ret
