@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import sys
+import os, sys
 
 from mercurial import util
 
@@ -35,14 +35,14 @@ class QuickOpDialog(QDialog):
         self._repoagent = repoagent
         self._cmdsession = cmdcore.nullCmdSession()
         self._cmddialog = cmdui.CmdSessionDialog(self)
-        repo = repoagent.rawRepo()
 
         # Handle rm alias
         if command == 'rm':
             command = 'remove'
         self.command = command
 
-        self.setWindowTitle(_('%s - hg %s') % (repo.displayname, command))
+        self.setWindowTitle(_('%s - hg %s')
+                            % (repoagent.displayName(), command))
         self.setWindowIcon(qtlib.geticon(ICONS[command]))
 
         layout = QVBoxLayout()
@@ -159,12 +159,13 @@ class QuickOpDialog(QDialog):
         self._cmdsession = sess = self._repoagent.runCommandSequence(ucmdlines,
                                                                      self)
         sess.commandFinished.connect(self.commandFinished)
-        sess.progressReceived.connect(self.statusbar.progress)
+        sess.progressReceived.connect(self.statusbar.setProgress)
         self._cmddialog.setSession(sess)
         self.bb.button(QDialogButtonBox.Ok).setEnabled(False)
 
     def commandFinished(self, ret):
         self.bb.button(QDialogButtonBox.Ok).setEnabled(True)
+        self.statusbar.clearProgress()
         if ret == 0:
             shlib.shell_notify(self.files)
             self.reject()
@@ -184,10 +185,8 @@ class QuickOpDialog(QDialog):
                                 _('No operation to perform'),
                                 parent=self)
             return
-        self.repo.bfstatus = True
         self.repo.lfstatus = True
         repostate = self.repo.status()
-        self.repo.bfstatus = False
         self.repo.lfstatus = False
         if self.command == 'remove':
             if not self.chk.isChecked():
@@ -292,4 +291,5 @@ def run(ui, repoagent, *pats, **opts):
         cmdline = [command] + pats
         return HeadlessQuickop(repoagent, cmdline)
     else:
+        os.chdir(repo.root)  # for scmutil.match() in StatusThread
         return QuickOpDialog(repoagent, command, pats, None)
