@@ -81,14 +81,15 @@ class RepoFilterBar(QToolBar):
 
     _allBranchesLabel = u'\u2605 ' + _('Show all') + u' \u2605'
 
-    def __init__(self, repo, parent=None):
+    def __init__(self, repoagent, parent=None):
         super(RepoFilterBar, self).__init__(parent)
         self.layout().setContentsMargins(0, 0, 0, 0)
-        self.setIconSize(QSize(16,16))
+        self.setIconSize(qtlib.smallIconSize())
         self.setFloatable(False)
         self.setMovable(False)
-        self._repo = repo
+        self._repoagent = repoagent
         self._permanent_queries = list(_permanent_queries)
+        repo = repoagent.rawRepo()
         username = repo.ui.config('ui', 'username')
         if username:
             self._permanent_queries.insert(0,
@@ -99,7 +100,7 @@ class RepoFilterBar(QToolBar):
         if not QFontMetrics(self.font()).inFont(QString(u'\u2605').at(0)):
             self._allBranchesLabel = u'*** %s ***' % _('Show all')
 
-        self.entrydlg = revset.RevisionSetQuery(repo, self)
+        self.entrydlg = revset.RevisionSetQuery(repoagent, self)
         self.entrydlg.progress.connect(self.progress)
         self.entrydlg.showMessage.connect(self.showMessage)
         self.entrydlg.queryIssued.connect(self.queryIssued)
@@ -115,7 +116,7 @@ class RepoFilterBar(QToolBar):
         le.selectionChanged.connect(self.selectionChanged)
         if hasattr(le, 'setPlaceholderText'): # Qt >= 4.7
             le.setPlaceholderText(_('### revision set query ###'))
-        combo.activated.connect(self.runQuery)
+        combo.activated.connect(self._revsetHistoryActivated)
 
         self._revsettypelabel = QLabel(le)
         self._revsettypetimer = QTimer(self, interval=200, singleShot=True)
@@ -180,6 +181,10 @@ class RepoFilterBar(QToolBar):
         self._initBranchFilter()
         self.refresh()
 
+    @property
+    def _repo(self):
+        return self._repoagent.rawRepo()
+
     def onClearButtonClicked(self):
         if self.revsetcombo.currentText():
             self.revsetcombo.clearEditText()
@@ -187,23 +192,16 @@ class RepoFilterBar(QToolBar):
             self.hide()
         self.clearRevisionSet.emit()
 
-    def setEnableFilter(self, enabled):
-        'Enable/disable the changing of the current filter'
-        self.revsetcombo.setEnabled(enabled)
-        self.clearBtn.setEnabled(enabled)
-        self.searchBtn.setEnabled(enabled)
-        self.editorBtn.setEnabled(enabled)
-        self.deleteBtn.setEnabled(enabled)
-        self._branchCombo.setEnabled(enabled)
-        self._branchLabel.setEnabled(enabled)
-        self.filterEnabled = enabled
-        self.showHiddenBtn.setEnabled(enabled)
-        self.showGraftSourceBtn.setEnabled(enabled)
-
     def selectionChanged(self):
         selection = self.revsetcombo.lineEdit().selectedText()
         self.deleteBtn.setEnabled(selection in self.revsethist)
 
+    @pyqtSlot()
+    def _revsetHistoryActivated(self):
+        # do not run query on focus out in the middle of history completion
+        if self.revsetcombo.hasFocus():
+            self.runQuery()
+
     def deleteFromHistory(self):
         selection = self.revsetcombo.lineEdit().selectedText()
         if selection not in self.revsethist:
@@ -216,7 +214,8 @@ class RepoFilterBar(QToolBar):
 
     def showEvent(self, event):
         super(RepoFilterBar, self).showEvent(event)
-        self.revsetcombo.setFocus()
+        if not event.spontaneous():
+            self.revsetcombo.setFocus()
 
     def eventFilter(self, watched, event):
         if watched is self.revsetcombo.lineEdit():
@@ -310,7 +309,7 @@ class RepoFilterBar(QToolBar):
         self.revsetcombo.setCurrentIndex(self.revsetcombo.findText(query))
 
     def loadSettings(self, s):
-        repoid = str(self._repo[0])
+        repoid = hglib.shortrepoid(self._repo)
         s.beginGroup('revset/' + repoid)
         self.entrydlg.restoreGeometry(s.value('geom').toByteArray())
         self.revsethist = list(s.value('queries').toStringList())
@@ -327,7 +326,7 @@ class RepoFilterBar(QToolBar):
 
     def saveSettings(self, s):
         try:
-            repoid = str(self._repo[0])
+            repoid = hglib.shortrepoid(self._repo)
         except EnvironmentError:
             return
         s.beginGroup('revset/' + repoid)
@@ -342,9 +341,8 @@ class RepoFilterBar(QToolBar):
 
     def _initBranchFilter(self):
         self._branchLabel = QToolButton(
-            text=_('Branch'), popupMode=QToolButton.MenuButtonPopup,
+            text=_('Branch'), popupMode=QToolButton.InstantPopup,
             statusTip=_('Display graph the named branch only'))
-        self._branchLabel.clicked.connect(self._branchLabel.showMenu)
         self._branchMenu = QMenu(self._branchLabel)
         self._abranchAction = self._branchMenu.addAction(
             _('Display only active branches'), self.refresh)
@@ -395,9 +393,9 @@ class RepoFilterBar(QToolBar):
                 for n in self._repo.heads()
                 if not self._repo[n].extra().get('close')]))
         elif self._cbranchAction.isChecked():
-            branches = sorted(self._repo.branchtags().keys())
+            branches = sorted(self._repo.branchmap())
         else:
-            branches = self._repo.namedbranches
+            branches = hglib.namedbranches(self._repo)
 
         # easy access to common branches (Python sorted() is stable)
         priomap = {self._repo.dirstate.branch(): -2, 'default': -1}
