@@ -17,14 +17,9 @@
 import binascii, os, re
 
 from mercurial import util, error
-from mercurial.util import propertycache
-from mercurial.context import workingctx
 
 from tortoisehg.util import hglib
-from tortoisehg.hgqt.graph import Graph
-from tortoisehg.hgqt.graph import revision_grapher
-from tortoisehg.hgqt.graph import LINE_TYPE_GRAFT
-from tortoisehg.hgqt import qtlib
+from tortoisehg.hgqt import filedata, graph, qtlib
 
 from tortoisehg.hgqt.i18n import _
 
@@ -38,9 +33,26 @@ mqpatchmimetype = 'application/thg-mqunappliedpatch'
 # TODO: Remove these two when we adopt GTK author color scheme
 COLORS = [ "blue", "darkgreen", "red", "green", "darkblue", "purple",
            "dodgerblue", Qt.darkYellow, "magenta", "darkred", "darkmagenta",
-           "darkcyan", "gray", ]
+           "darkcyan", ]
 COLORS = [str(QColor(x).name()) for x in COLORS]
 
+# pick names from "hg help templating" if any
+GraphColumn = 0
+RevColumn = 1
+BranchColumn = 2
+DescColumn = 3
+AuthorColumn = 4
+TagsColumn = 5
+LatestTagColumn = 6
+NodeColumn = 7
+AgeColumn = 8
+LocalDateColumn = 9
+UtcDateColumn = 10
+ChangesColumn = 11
+ConvertedColumn = 12
+PhaseColumn = 13
+FileColumn = 14
+
 COLUMNHEADERS = (
     ('Graph', _('Graph', 'column header')),
     ('Rev', _('Rev', 'column header')),
@@ -56,12 +68,14 @@ COLUMNHEADERS = (
     ('Changes', _('Changes', 'column header')),
     ('Converted', _('Converted From', 'column header')),
     ('Phase', _('Phase', 'column header')),
+    ('Filename', _('Filename', 'column header')),
     )
+ALLCOLUMNS = tuple(name for name, _text in COLUMNHEADERS)
 
 UNAPPLIED_PATCH_COLOR = '#999999'
 HIDDENREV_COLOR = '#666666'
 
-GraphRole = Qt.UserRole + 0
+GraphNodeRole = Qt.UserRole + 0
 
 def get_color(n, ignore=()):
     """
@@ -74,16 +88,6 @@ def get_color(n, ignore=()):
         colors = COLORS
     return colors[n % len(colors)]
 
-def get_style(line_type, active):
-    if line_type == LINE_TYPE_GRAFT:
-        return Qt.DashLine
-    return Qt.SolidLine
-
-def get_width(line_type, active):
-    if line_type == LINE_TYPE_GRAFT or not active:
-        return 1
-    return 2
-
 def _parsebranchcolors(value):
     r"""Parse tortoisehg.branchcolors setting
 
@@ -124,6 +128,38 @@ def _parsebranchcolors(value):
         colors.append((key, val))
     return colors
 
+def _renderlabels(labels, margin=2):
+    if not labels:
+        return
+    font = QApplication.font('QAbstractItemView')
+    fm = QFontMetrics(font, None)  # screen-compatible (i.e. QPixmap) metrics
+    twidths = [fm.width(t) for t, _s in labels]
+    th = fm.height()
+
+    padw = 2
+    padh = 1  # may overwrite horizontal frame to fit row
+
+    pix = QPixmap(sum(twidths) + len(labels) * (2 * padw + margin) - margin,
+                  th + 2 * padh)
+    pix.fill(Qt.transparent)
+    painter = QPainter(pix)
+    painter.setFont(font)
+    x = 0
+    for (text, style), tw in zip(labels, twidths):
+        lw = tw + 2 * padw
+        lh = th + 2 * padh
+        # draw bevel, background and text in order
+        bg = qtlib.getbgcoloreffect(style)
+        painter.fillRect(x, 0, lw, lh, bg.darker(110))
+        painter.fillRect(x + 1, 1, lw - 2, lh - 2, bg.lighter(110))
+        painter.fillRect(x + 2, 2, lw - 4, lh - 4, bg)
+        painter.setPen(qtlib.gettextcoloreffect(style))
+        painter.drawText(x + padw, padh, tw, th, 0, text)
+        x += lw + margin
+    painter.end()
+    return pix
+
+
 class HgRepoListModel(QAbstractTableModel):
     """
     Model used for displaying the revisions of a Hg *local* repository
@@ -132,17 +168,13 @@ class HgRepoListModel(QAbstractTableModel):
     filled = pyqtSignal()
     loaded = pyqtSignal()
 
-    _allcolumns = tuple(h[0] for h in COLUMNHEADERS)
-    _allcolnames = dict(COLUMNHEADERS)
+    _defaultcolumns = ('Graph', 'Rev', 'Branch', 'Description', 'Author',
+                       'Age', 'Tags', 'Phase')
 
-    _columns = ('Graph', 'Rev', 'Branch', 'Description', 'Author', 'Age', 'Tags', 'Phase',)
-    _columnfonts = {'Node': QFont("Monospace"),
-                    'Converted': QFont("Monospace")}
-    _stretchs = {'Description': 1, }
     _mqtags = ('qbase', 'qtip', 'qparent')
 
-    def __init__(self, repo, cfgname, branch, revset, rfilter, parent,
-            showhidden=False, allparents=False, showgraftsource=True):
+    def __init__(self, repo, branch, revset, rfilter, parent,
+                 allparents=False, showgraftsource=True):
         """
         repo is a hg repo instance
         """
@@ -150,19 +182,15 @@ class HgRepoListModel(QAbstractTableModel):
         self._cache = []
         self.graph = None
         self.timerHandle = None
-        self.dotradius = 8
-        self.rowheight = 20
         self.rowcount = 0
         self.repo = repo
-        self.revset = revset
+        self.revset = frozenset(revset)
         self.filterbyrevset = rfilter
         self.unicodestar = True
         self.unicodexinabox = True
-        self.cfgname = cfgname
         self.latesttags = {-1: 'null'}
         self.fullauthorname = False
         self.filterbranch = branch  # unicode
-        self.showhidden = showhidden
         self.allparents = allparents
         self.showgraftsource = showgraftsource
 
@@ -173,7 +201,6 @@ class HgRepoListModel(QAbstractTableModel):
         if repo:
             self.initBranchColors()
             self.reloadConfig()
-            self.updateColumns()
             self._initGraph()
 
     def initBranchColors(self):
@@ -190,7 +217,7 @@ class HgRepoListModel(QAbstractTableModel):
         # Then assign colors to all branches in alphabetical order
         # Note that re-assigning the color to the default branch
         # is not expensive
-        for branch in sorted(self.repo.branchtags().keys()):
+        for branch in sorted(self.repo.branchmap()):
             self.namedbranch_color(branch)
 
     def setBranch(self, branch, allparents=False):
@@ -198,42 +225,33 @@ class HgRepoListModel(QAbstractTableModel):
         self.allparents = allparents
         self._initGraph()
 
-    def setShowHidden(self, visible):
-        self.showhidden = visible
-        self._initGraph()
-
     def setShowGraftSource(self, visible):
         self.showgraftsource = visible
         self._initGraph()
 
     def _initGraph(self):
-        branch = self.filterbranch
-        allparents = self.allparents
-        showhidden = self.showhidden
-        showgraftsource = self.showgraftsource
         self.invalidateCache()
+        opts = {
+            'branch': hglib.fromunicode(self.filterbranch),
+            'showgraftsource': self.showgraftsource,
+            }
         if self.revset and self.filterbyrevset:
-            grapher = revision_grapher(self.repo,
-                                       branch=hglib.fromunicode(branch),
-                                       revset=self.revset,
-                                       showhidden=showhidden,
-                                       showgraftsource=showgraftsource)
-            self.graph = Graph(self.repo, grapher, include_mq=False)
+            opts['revset'] = self.revset
+            opts['showfamilyline'] = \
+                self.repo.ui.configbool('tortoisehg', 'showfamilyline', True)
+            grapher = graph.revision_grapher(self.repo, opts)
+            self.graph = graph.Graph(self.repo, grapher, include_mq=False)
         else:
-            grapher = revision_grapher(self.repo,
-                                       branch=hglib.fromunicode(branch),
-                                       allparents=allparents,
-                                       showhidden=showhidden,
-                                       showgraftsource=showgraftsource)
-            self.graph = Graph(self.repo, grapher, include_mq=True)
+            opts['allparents'] = self.allparents
+            grapher = graph.revision_grapher(self.repo, opts)
+            self.graph = graph.Graph(self.repo, grapher, include_mq=True)
         self.rowcount = 0
         self.layoutChanged.emit()
-        self.ensureBuilt(row=0)
         self.showMessage.emit('')
         QTimer.singleShot(0, self, SIGNAL('filled()'))
 
     def setRevset(self, revset):
-        self.revset = revset
+        self.revset = frozenset(revset)
         self.invalidateCache()
 
     def reloadConfig(self):
@@ -242,23 +260,6 @@ class HgRepoListModel(QAbstractTableModel):
         self.authorcolor = _ui.configbool('tortoisehg', 'authorcolor')
         self.fullauthorname = _ui.configbool('tortoisehg', 'fullauthorname')
 
-    def updateColumns(self):
-        s = QSettings()
-        cols = s.value(self.cfgname + '/columns').toStringList()
-        cols = [str(col) for col in cols]
-        # Fixup older names for columns
-        if 'Log' in cols:
-            cols[cols.index('Log')] = 'Description'
-            s.setValue(self.cfgname + '/columns', cols)
-        if 'ID' in cols:
-            cols[cols.index('ID')] = 'Rev'
-            s.setValue(self.cfgname + '/columns', cols)
-        validcols = [col for col in cols if col in self._allcolumns]
-        if validcols:
-            self._columns = tuple(validcols)
-            self.invalidateCache()
-            self.layoutChanged.emit()
-
     def invalidate(self):
         self.reloadConfig()
         self.invalidateCache()
@@ -267,35 +268,29 @@ class HgRepoListModel(QAbstractTableModel):
     def branch(self):
         return self.filterbranch
 
-    def ensureBuilt(self, rev=None, row=None):
+    def canFetchMore(self, parent):
+        if parent.isValid():
+            return False
+        return not self.graph.isfilled()
+
+    def fetchMore(self, parent):
+        if parent.isValid() or self.graph.isfilled():
+            return
+        self.graph.build_nodes(self.fill_step)
+        self.updateRowCount()
+
+    def ensureBuilt(self, rev):
         """
         Make sure rev data is available (graph element created).
 
         """
-        if self.graph.isfilled():
+        if (self.graph.isfilled()
+            # working rev must exist once build_nodes() is invoked
+            or (len(self.graph) > 0 and rev is None)
+            or (len(self.graph) > 0 and self.graph[-1].rev <= rev)):
             return
-        required = 0
-        buildrev = rev
-        n = len(self.graph)
-        if rev is not None:
-            if n and self.graph[-1].rev <= rev:
-                buildrev = None
-            else:
-                required = self.fill_step/2
-        elif row is not None and row > (n - self.fill_step / 2):
-            required = row - n + self.fill_step
-        if required or buildrev:
-            self.graph.build_nodes(nnodes=required, rev=buildrev)
-            self.updateRowCount()
-
-        if self.rowcount >= len(self.graph):
-            return  # no need to update row count
-        if row and row > self.rowcount:
-            # asked row was already built, but views where not aware of this
-            self.updateRowCount()
-        elif rev is not None and rev <= self.graph[self.rowcount].rev:
-            # asked rev was already built, but views where not aware of this
-            self.updateRowCount()
+        self.graph.build_nodes(self.fill_step / 2, rev)
+        self.updateRowCount()
 
     def loadall(self):
         self.timerHandle = self.startTimer(1)
@@ -327,44 +322,40 @@ class HgRepoListModel(QAbstractTableModel):
             self.rowcount = newlen
             self.endInsertRows()
 
-    def rowCount(self, parent):
+    def rowCount(self, parent=QModelIndex()):
         if parent.isValid():
             return 0
         return self.rowcount
 
-    def columnCount(self, parent):
+    def columnCount(self, parent=QModelIndex()):
         if parent.isValid():
             return 0
-        return len(self._columns)
+        return len(ALLCOLUMNS) - 1  # no FileColumn
 
-    def maxWidthValueForColumn(self, col):
+    def maxWidthValueForColumn(self, column):
         if self.graph is None:
             return 'XXXX'
-        column = self._columns[col]
-        if column == 'Rev':
+        if column == RevColumn:
             return '8' * len(str(len(self.repo))) + '+'
-        if column == 'Node':
+        if column == NodeColumn:
             return '8' * 12 + '+'
-        if column in ('LocalTime', 'UTCTime'):
+        if column in (LocalDateColumn, UtcDateColumn):
             return hglib.displaytime(util.makedate())
-        if column in ('Tags', 'Latest tags'):
+        if column in (TagsColumn, LatestTagColumn):
             try:
                 return sorted(self.repo.tags().keys(), key=lambda x: len(x))[-1][:10]
             except IndexError:
                 pass
-        if column == 'Branch':
+        if column == BranchColumn:
             try:
-                return sorted(self.repo.branchtags().keys(), key=lambda x: len(x))[-1]
+                return sorted(self.repo.branchmap(), key=lambda x: len(x))[-1]
             except IndexError:
                 pass
-        if column == 'Filename':
+        if column == FileColumn:
             return self.filename
-        if column == 'Graph':
-            res = self.col2x(self.graph.max_cols)
-            return min(res, 150)
-        if column == 'Changes':
+        if column == ChangesColumn:
             return 'Changes'
-        # Fall through for Description
+        # Fall through for DescColumn
         return None
 
     def user_color(self, user):
@@ -380,165 +371,32 @@ class HgRepoListModel(QAbstractTableModel):
             self._branch_colors[branch] = get_color(len(self._branch_colors))
         return self._branch_colors[branch]
 
-    def col2x(self, col):
-        return 2 * self.dotradius * col + self.dotradius/2 + 8
-
-    def graphctx(self, ctx, gnode):
-        w = self.col2x(gnode.cols) + 10
-        h = self.rowheight
-
-        pix = QPixmap(w, h)
-        pix.fill(QColor(0,0,0,0))
-        painter = QPainter(pix)
-        try:
-            self._drawgraphctx(painter, pix, ctx, gnode)
-        finally:
-            painter.end()
-        return QVariant(pix)
-
-    def _drawgraphctx(self, painter, pix, ctx, gnode):
-        revset = self.revset
-        h = pix.height()
-        dot_y = h / 2
-
-        painter.setRenderHint(QPainter.Antialiasing)
-
-        pen = QPen(Qt.blue)
-        pen.setWidth(2)
-        painter.setPen(pen)
-
-        lpen = QPen(pen)
-        lpen.setColor(Qt.black)
-        painter.setPen(lpen)
-        if revset:
-            def isactive(start, end, color, line_type, children, rev):
-                return rev in revset and util.any(r in revset for r in children)
-        else:
-            def isactive(start, end, color, line_type, children, rev):
-                return True
-
-        for y1, y4, lines in ((dot_y, dot_y + h, gnode.bottomlines),
-                              (dot_y - h, dot_y, gnode.toplines)):
-            y2 = y1 + 1 * (y4 - y1)/4
-            ymid = (y1 + y4)/2
-            y3 = y1 + 3 * (y4 - y1)/4
-
-            lines = sorted((isactive(*l), l) for l in lines)
-
-            for active, (start, end, color, line_type, children, rev) in lines:
-                lpen = QPen(pen)
-                lpen.setColor(QColor(active and get_color(color) or "gray"))
-                lpen.setStyle(get_style(line_type, active))
-                lpen.setWidth(get_width(line_type, active))
-                painter.setPen(lpen)
-                x1 = self.col2x(start)
-                x2 = self.col2x(end)
-                path = QPainterPath()
-                path.moveTo(x1, y1)
-                path.cubicTo(x1, y2,
-                             x1, y2,
-                             (x1 + x2)/2, ymid)
-                path.cubicTo(x2, y3,
-                             x2, y3,
-                             x2, y4)
-                painter.drawPath(path)
-
-        # Draw node
-        if revset and gnode.rev not in revset:
-            dot_color = QColor("gray")
-            radius = self.dotradius * 0.8
-        else:
-            dot_color = QColor(self.namedbranch_color(ctx.branch()))
-            radius = self.dotradius
-        dotcolor = dot_color.lighter()
-        pencolor = dot_color.darker()
-        truewhite = QColor("white")
-        white = QColor("white")
-        fillcolor = gnode.rev is None and white or dotcolor
-
-        pen = QPen(pencolor)
-        pen.setWidthF(1.5)
-        painter.setPen(pen)
-
-        centre_x = self.col2x(gnode.x)
-        centre_y = h/2
-
-        def circle(r):
-            rect = QRectF(centre_x - r,
-                          centre_y - r,
-                          2 * r, 2 * r)
-            painter.drawEllipse(rect)
-
-        def closesymbol(s):
-            rect_ = QRectF(centre_x - 1.5 * s, centre_y - 0.5 * s, 3 * s, s)
-            painter.drawRect(rect_)
-
-        def diamond(r):
-            poly = QPolygonF([QPointF(centre_x - r, centre_y),
-                              QPointF(centre_x, centre_y - r),
-                              QPointF(centre_x + r, centre_y),
-                              QPointF(centre_x, centre_y + r),
-                              QPointF(centre_x - r, centre_y),])
-            painter.drawPolygon(poly)
-
-        hiddenrev = ctx.hidden()
-        if hiddenrev:
-            painter.setBrush(truewhite)
-            white.setAlpha(64)
-            fillcolor.setAlpha(64)
-        if ctx.thgmqappliedpatch():  # diamonds for patches
-            symbolsize = radius / 1.5
-            if hiddenrev:
-                diamond(symbolsize)
-            if ctx.thgwdparent():
-                painter.setBrush(white)
-                diamond(2 * 0.9 * symbolsize)
-            painter.setBrush(fillcolor)
-            diamond(symbolsize)
-        elif ctx.thgmqunappliedpatch():
-            symbolsize = radius / 1.5
-            if hiddenrev:
-                diamond(symbolsize)
-            patchcolor = QColor('#dddddd')
-            painter.setBrush(patchcolor)
-            painter.setPen(patchcolor)
-            diamond(symbolsize)
-        elif ctx.extra().get('close'):
-            symbolsize = 0.5 * radius
-            if hiddenrev:
-                closesymbol(symbolsize)
-            painter.setBrush(fillcolor)
-            closesymbol(symbolsize)
-        else:  # circles for normal revisions
-            symbolsize = 0.5 * radius
-            if hiddenrev:
-                circle(symbolsize)
-            if ctx.thgwdparent():
-                painter.setBrush(white)
-                circle(0.9 * radius)
-            painter.setBrush(fillcolor)
-            circle(symbolsize)
-
     def invalidateCache(self):
         self._cache = []
-        for a in ('_roleoffsets',):
-            if hasattr(self, a):
-                delattr(self, a)
 
-    @propertycache
-    def _roleoffsets(self):
-        return {Qt.DisplayRole : 0,
-                Qt.ForegroundRole : len(self._columns),
-                GraphRole : len(self._columns) * 2}
-
-    def data(self, index, role):
+    def data(self, index, role=Qt.DisplayRole):
         if not index.isValid():
             return nullvariant
-        # font is not cached in self._cache since it is equal for all rows
+        gnode = self.graph[index.row()]
+        if role == Qt.DisplayRole:
+            if index.column() == FileColumn:
+                return hglib.tounicode(gnode.extra[0])
         if role == Qt.FontRole:
-            column = self._columns[index.column()]
-            return self._columnfonts.get(column, nullvariant)
-        if role not in self._roleoffsets:
+            if index.column() in (NodeColumn, ConvertedColumn):
+                return QFont("Monospace")
+            if index.column() == DescColumn and gnode.wdparent:
+                font = QApplication.font('QAbstractItemView')
+                font.setBold(True)
+                return font
+        if role == Qt.ForegroundRole:
+            if (gnode.shape == graph.NODE_SHAPE_UNAPPLIEDPATCH
+                and index.column() != DescColumn):
+                return QColor(UNAPPLIED_PATCH_COLOR)
+            if gnode.hidden and index.column() != GraphColumn:
+                return QColor(HIDDENREV_COLOR)
+        if role == GraphNodeRole:
+            return gnode
+        if (role, index.column()) not in self._cacheindexmap:
             return nullvariant
         # repo may be changed while reading in case of postpull=rebase for
         # example, and result in RevlogError. (issue #429)
@@ -554,78 +412,65 @@ class HgRepoListModel(QAbstractTableModel):
 
     def safedata(self, index, role):
         row = index.row()
-        self.ensureBuilt(row=row)
         graphlen = len(self.graph)
         cachelen = len(self._cache)
         if graphlen > cachelen:
-            self._cache.extend([None,] * (graphlen-cachelen))
+            self._cache.extend([None] * len(self._cacheindexmap)
+                               for _i in xrange(graphlen - cachelen))
         data = self._cache[row]
-        if data is None:
-            data = [None,] * (self._roleoffsets[GraphRole]+1)
-        column = self._columns[index.column()]
-        offset = self._roleoffsets[role]
-        if role == GraphRole:
-            if column != 'Graph':
-                return nullvariant
-            if data[offset] is None:
-                gnode = self.graph[row]
-                ctx = self.repo.changectx(gnode.rev)
-                data[offset] = self.graphctx(ctx, gnode)
-                self._cache[row] = data
-            return data[offset]
-        else:
-            idx = index.column() + offset
-            if data[idx] is None:
-                try:
-                    result = self.rawdata(row, column, role)
-                except util.Abort:
-                    result = nullvariant
-                data[idx] = result
-                self._cache[row] = data
-            return data[idx]
+        idx = self._cacheindexmap[role, index.column()]
+        if data[idx] is None:
+            try:
+                result = self.rawdata(index, role)
+            except util.Abort:
+                result = nullvariant
+            data[idx] = result
+        return data[idx]
 
-    def rawdata(self, row, column, role):
+    def rawdata(self, index, role):
+        row = index.row()
+        column = index.column()
         gnode = self.graph[row]
         ctx = self.repo.changectx(gnode.rev)
 
         if role == Qt.DisplayRole:
-            text = self._columnmap[column](self, ctx, gnode)
+            text = self._columnmap[column](self, ctx)
             if not isinstance(text, (QString, unicode)):
                 text = hglib.tounicode(text)
             return QVariant(text)
         elif role == Qt.ForegroundRole:
-            if ctx.thgmqunappliedpatch():
-                return QColor(UNAPPLIED_PATCH_COLOR)
-            if ctx.hidden():
-                return QColor(HIDDENREV_COLOR)
-            if column == 'Author':
+            if column == AuthorColumn:
                 if self.authorcolor:
                     return QVariant(QColor(self.user_color(ctx.user())))
                 return nullvariant
-            if column == 'Branch':
+            if column in (GraphColumn, BranchColumn):
                 return QVariant(QColor(self.namedbranch_color(ctx.branch())))
+        elif role == Qt.DecorationRole and column == DescColumn:
+            return QVariant(self._renderrevlabels(ctx))
+        elif role == Qt.DecorationRole and column == ChangesColumn:
+            return QVariant(self._renderchanges(ctx))
         return nullvariant
 
     def flags(self, index):
+        flags = super(HgRepoListModel, self).flags(index)
         if not index.isValid():
-            return Qt.ItemFlags(0)
+            return flags
         row = index.row()
-        self.ensureBuilt(row=row)
-        if row >= len(self.graph):
-            return Qt.ItemFlags(0)
+        if row >= len(self.graph) and not self.repo.ui.debugflag:
+            # TODO: should not happen; internal data went wrong (issue #754)
+            return Qt.NoItemFlags
         gnode = self.graph[row]
-        ctx = self.repo.changectx(gnode.rev)
+        if not self.isActiveRev(gnode.rev):
+            return Qt.NoItemFlags
+        if gnode.shape == graph.NODE_SHAPE_UNAPPLIEDPATCH:
+            flags |= Qt.ItemIsDragEnabled | Qt.ItemIsDropEnabled
+        if gnode.rev is None:
+            flags |= Qt.ItemIsDropEnabled
+        return flags
 
-        dragflags = Qt.ItemFlags(0)
-        if ctx.thgmqunappliedpatch():
-            dragflags = Qt.ItemIsDragEnabled | Qt.ItemIsDropEnabled
-        if isinstance(ctx, workingctx):
-            dragflags |= Qt.ItemIsDropEnabled
-        if not self.revset:
-            return Qt.ItemIsSelectable | Qt.ItemIsEnabled | dragflags
-        if ctx.rev() not in self.revset:
-            return Qt.ItemFlags(0)
-        return Qt.ItemIsSelectable | Qt.ItemIsEnabled | dragflags
+    def isActiveRev(self, rev):
+        """True if the specified rev is not excluded by revset"""
+        return not self.revset or rev in self.revset
 
     def mimeTypes(self):
         return QStringList([mqpatchmimetype])
@@ -662,41 +507,25 @@ class HgRepoListModel(QAbstractTableModel):
         else:
             destpatch = None  # next to working rev
 
-        self.repo.incrementBusyCount()
-        try:
-            hglib.movemqpatches(self.repo, destpatch, dragpatches)
-        finally:
-            self.repo.decrementBusyCount()
+        hglib.movemqpatches(self.repo, destpatch, dragpatches)
+        repoagent = self.repo._pyqtobj  # TODO
+        repoagent.pollStatus()
         return True
 
-    def headerData(self, section, orientation, role):
-        if orientation == Qt.Horizontal:
-            if role == Qt.DisplayRole:
-                return QVariant(self._allcolnames[self._columns[section]])
-        return nullvariant
-
-    def rowFromRev(self, rev):
-        row = self.graph.index(rev)
-        if row == -1:
-            row = None
-        return row
+    def headerData(self, section, orientation, role=Qt.DisplayRole):
+        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+            return COLUMNHEADERS[section][1]
 
     def indexFromRev(self, rev):
         if self.graph is None:
-            return None
-        self.ensureBuilt(rev=rev)
-        row = self.rowFromRev(rev)
-        if row is not None:
+            return QModelIndex()
+        self.ensureBuilt(rev)
+        row = self.graph.index(rev)
+        if row >= 0:
             return self.index(row, 0)
-        return None
+        return QModelIndex()
 
-    def clear(self):
-        'empty the list'
-        self.graph = None
-        self.datacache = {}
-        self.layoutChanged.emit()
-
-    def getbranch(self, ctx, gnode):
+    def getbranch(self, ctx):
         b = hglib.tounicode(ctx.branch())
         if ctx.extra().get('close'):
             if self.unicodexinabox:
@@ -705,7 +534,7 @@ class HgRepoListModel(QAbstractTableModel):
                 b += u'--'
         return b
 
-    def getlatesttags(self, ctx, gnode):
+    def getlatesttags(self, ctx):
         rev = ctx.rev()
         todo = [rev]
         repo = self.repo
@@ -714,7 +543,8 @@ class HgRepoListModel(QAbstractTableModel):
             if rev in self.latesttags:
                 continue
             ctx = repo[rev]
-            tags = [t for t in ctx.tags() if repo.tagtype(t) == 'global']
+            tags = [t for t in ctx.tags()
+                    if repo.tagtype(t) and repo.tagtype(t) != 'local']
             if tags:
                 self.latesttags[rev] = ':'.join(sorted(tags))
                 continue
@@ -732,13 +562,13 @@ class HgRepoListModel(QAbstractTableModel):
             self.latesttags[rev] = ptag
         return self.latesttags[rev]
 
-    def gettags(self, ctx, gnode):
+    def gettags(self, ctx):
         if ctx.rev() is None:
             return ''
         tags = [t for t in ctx.tags() if t not in self._mqtags]
         return hglib.tounicode(','.join(tags))
 
-    def getrev(self, ctx, gnode):
+    def getrev(self, ctx):
         rev = ctx.rev()
         if type(rev) is int:
             return str(rev)
@@ -747,7 +577,7 @@ class HgRepoListModel(QAbstractTableModel):
         else:
             return ''
 
-    def getauthor(self, ctx, gnode):
+    def getauthor(self, ctx):
         try:
             user = ctx.user()
             if not self.fullauthorname:
@@ -756,38 +586,34 @@ class HgRepoListModel(QAbstractTableModel):
         except error.Abort:
             return _('Mercurial User')
 
-    def getlog(self, ctx, gnode):
+    def getlog(self, ctx):
         if ctx.rev() is None:
-            msg = None
             if self.unicodestar:
                 # The Unicode symbol is a black star:
-                msg = u'\u2605 ' + _('Working Directory') + u' \u2605'
+                return u'\u2605 ' + _('Working Directory') + u' \u2605'
             else:
-                msg = '*** ' + _('Working Directory') + ' ***'
+                return '*** ' + _('Working Directory') + ' ***'
+        if self.repo.ui.configbool('tortoisehg', 'longsummary'):
+            limit = 0x7fffffff  # unlimited (elide it by view)
+        else:
+            limit = None  # first line
+        return hglib.longsummary(ctx.description(), limit)
 
+    def _renderrevlabels(self, ctx):
+        labels = []
+        if ctx.rev() is None:
             for pctx in ctx.parents():
-                if self.repo._branchheads and pctx.node() not in self.repo._branchheads:
-                    text = _('Not a head revision!')
-                    msg += " " + qtlib.markup(text, fg='red', weight='bold')
+                branchheads = hglib.branchheads(self.repo)
+                if branchheads and pctx.node() not in branchheads:
+                    labels.append((_('Not a head revision!'), 'log.warning'))
+            return _renderlabels(labels)
 
-            return msg
-
-        msg = ctx.longsummary()
+        if ctx.thgbranchhead():
+            labels.append((hglib.tounicode(ctx.branch()), 'log.branch'))
 
         if ctx.thgmqunappliedpatch():
-            effects = qtlib.geteffect('log.unapplied_patch')
-            text = qtlib.applyeffects(' %s ' % ctx._patchname, effects)
-            # qtlib.markup(msg, fg=UNAPPLIED_PATCH_COLOR)
-            msg = qtlib.markup(msg)
-            return hglib.tounicode(text + ' ') + msg
-        if ctx.hidden():
-            return qtlib.markup(msg, fg=HIDDENREV_COLOR)
-
-        parts = []
-        if ctx.thgbranchhead():
-            branchu = hglib.tounicode(ctx.branch())
-            effects = qtlib.geteffect('log.branch')
-            parts.append(qtlib.applyeffects(u' %s ' % branchu, effects))
+            style = 'log.unapplied_patch'
+            labels.append((hglib.tounicode(ctx._patchname), style))
 
         for mark in ctx.bookmarks():
             style = 'log.bookmark'
@@ -795,45 +621,30 @@ class HgRepoListModel(QAbstractTableModel):
                 bn = self.repo._bookmarks[self.repo._bookmarkcurrent]
                 if bn in self.repo.dirstate.parents():
                     style = 'log.curbookmark'
-            marku = hglib.tounicode(mark)
-            effects = qtlib.geteffect(style)
-            parts.append(qtlib.applyeffects(u' %s ' % marku, effects))
+            labels.append((hglib.tounicode(mark), style))
 
         for tag in ctx.thgtags():
             if self.repo.thgmqtag(tag):
                 style = 'log.patch'
             else:
                 style = 'log.tag'
-            tagu = hglib.tounicode(tag)
-            effects = qtlib.geteffect(style)
-            parts.append(qtlib.applyeffects(u' %s ' % tagu, effects))
+            labels.append((hglib.tounicode(tag), style))
 
-        if msg:
-            if ctx.thgwdparent():
-                msg = qtlib.markup(msg, weight='bold')
-            else:
-                msg = qtlib.markup(msg)
-            parts.append(hglib.tounicode(msg))
+        return _renderlabels(labels)
 
-        return ' '.join(parts)
-
-    def getchanges(self, ctx, gnode):
+    def _renderchanges(self, ctx):
         """Return the MAR status for the given ctx."""
-        changes = []
+        labels = []
         M, A, R = ctx.changesToParent(0)
-        def addtotal(files, style):
-            effects = qtlib.geteffect(style)
-            text = qtlib.applyeffects(' %s ' % len(files), effects)
-            changes.append(text)
         if A:
-            addtotal(A, 'log.added')
+            labels.append((str(len(A)), 'log.added'))
         if M:
-            addtotal(M, 'log.modified')
+            labels.append((str(len(M)), 'log.modified'))
         if R:
-            addtotal(R, 'log.removed')
-        return ''.join(changes)
+            labels.append((str(len(R)), 'log.removed'))
+        return _renderlabels(labels, margin=0)
 
-    def getconv(self, ctx, gnode):
+    def getconv(self, ctx):
         if ctx.rev() is not None:
             extra = ctx.extra()
             cvt = extra.get('convert_revision', '')
@@ -851,7 +662,7 @@ class HgRepoListModel(QAbstractTableModel):
                 return cvt
         return ''
 
-    def getphase(self, ctx, gnode):
+    def getphase(self, ctx):
         if ctx.rev() is None:
             return ''
         try:
@@ -860,19 +671,87 @@ class HgRepoListModel(QAbstractTableModel):
             return 'draft'
 
     _columnmap = {
-        'Rev':      getrev,
-        'Node':     lambda self, ctx, gnode: str(ctx),
-        'Graph':    lambda self, ctx, gnode: "",
-        'Description': getlog,
-        'Author':   getauthor,
-        'Tags':     gettags,
-        'Latest tags':     getlatesttags,
-        'Branch':   getbranch,
-        'Filename': lambda self, ctx, gnode: gnode.extra[0],
-        'Age':      lambda self, ctx, gnode: hglib.age(ctx.date()).decode('utf-8'),
-        'LocalTime':lambda self, ctx, gnode: hglib.displaytime(ctx.date()),
-        'UTCTime':  lambda self, ctx, gnode: hglib.utctime(ctx.date()),
-        'Changes':  getchanges,
-        'Converted': getconv,
-        'Phase':    getphase,
-    }
+        RevColumn: getrev,
+        BranchColumn: getbranch,
+        DescColumn: getlog,
+        AuthorColumn: getauthor,
+        TagsColumn: gettags,
+        LatestTagColumn: getlatesttags,
+        NodeColumn: lambda self, ctx: str(ctx),
+        AgeColumn: lambda self, ctx: hglib.age(ctx.date()).decode('utf-8'),
+        LocalDateColumn: lambda self, ctx: hglib.displaytime(ctx.date()),
+        UtcDateColumn: lambda self, ctx: hglib.utctime(ctx.date()),
+        ConvertedColumn: getconv,
+        PhaseColumn: getphase,
+        }
+
+    # (role, column): index in _cache[row]
+    _cacheindexmap = dict((k, i) for i, k in enumerate(
+        [(Qt.DisplayRole, c) for c in _columnmap]
+        + [(Qt.ForegroundRole, GraphColumn),
+           (Qt.ForegroundRole, BranchColumn),
+           (Qt.ForegroundRole, AuthorColumn),
+           (Qt.DecorationRole, DescColumn),
+           (Qt.DecorationRole, ChangesColumn)]))
+
+
+class FileRevModel(HgRepoListModel):
+    """
+    Model used to manage the list of revisions of a file, in file
+    viewer of in diff-file viewer dialogs.
+    """
+
+    _defaultcolumns = ('Graph', 'Rev', 'Branch', 'Description', 'Author',
+                       'Age', 'Filename')
+
+    def __init__(self, repo, filename, parent=None):
+        """
+        data is a HgHLRepo instance
+        """
+        self.filename = filename
+        HgRepoListModel.__init__(self, repo, '', [], False, parent)
+
+    def _initGraph(self):
+        grapher = graph.filelog_grapher(self.repo, self.filename)
+        self.graph = graph.Graph(self.repo, grapher)
+        QTimer.singleShot(0, self, SIGNAL('filled()'))
+
+    def columnCount(self, parent=QModelIndex()):
+        if parent.isValid():
+            return 0
+        return len(ALLCOLUMNS)
+
+    def indexLinkedFromRev(self, rev):
+        """Index for the last changed revision before the specified revision
+
+        This does not follow renames.
+        """
+        # as of Mercurial 2.6, workingfilectx.linkrev() does not work, and
+        # this model has no virtual working-dir revision.
+        if rev is None:
+            rev = '.'
+        try:
+            fctx = self.repo[rev][self.filename]
+        except error.LookupError:
+            return QModelIndex()
+        return self.indexFromRev(fctx.linkrev())
+
+    def fileData(self, index, baseindex=QModelIndex()):
+        """Displayable file data at the given index; baseindex specifies the
+        revision where status is calculated from"""
+        row = index.row()
+        if not index.isValid() or row < 0 or row >= len(self.graph):
+            return filedata.createNullData(self.repo)
+        rev = self.graph[row].rev
+        ctx = self.repo.changectx(rev)
+        if baseindex.isValid():
+            prev = self.graph[baseindex.row()].rev
+            pctx = self.repo.changectx(prev)
+        else:
+            pctx = ctx.p1()
+        filename = self.graph.filename(rev)
+        if filename in pctx:
+            status = 'M'
+        else:
+            status = 'A'
+        return filedata.createFileData(ctx, pctx, filename, status)
