@@ -61,12 +61,12 @@ def readXml(source, rootElementName):
                   "(was looking for %s)" % (ele, rootElementName)
             return
     if xr.hasError():
-        print str(xr.errorString())
+        print hglib.fromunicode(xr.errorString(), 'replace')
     if xr.readNextStartElement():
         itemread = repotreeitem.undumpObject(xr)
         xr.skipCurrentElement()
     if xr.hasError():
-        print str(xr.errorString())
+        print hglib.fromunicode(xr.errorString(), 'replace')
     return itemread
 
 def iterRepoItemFromXml(source):
@@ -315,7 +315,7 @@ class RepoTreeModel(QAbstractItemModel):
                 for e in getRepoItemList(self.rootItem, standalone)]
 
     def _indexFromItem(self, item, column=0):
-        if item:
+        if item and item is not self.rootItem:
             return self.createIndex(item.row(), column, item)
         else:
             return QModelIndex()
@@ -333,6 +333,22 @@ class RepoTreeModel(QAbstractItemModel):
         ri.appendChild(repotreeitem.RepoGroupItem(name, ri))
         self.endInsertRows()
 
+    def itemPath(self, index):
+        """Virtual path of the item at the given index"""
+        if index.isValid():
+            item = index.internalPointer()
+        else:
+            item = self.rootItem
+        return repotreeitem.itempath(item)
+
+    def indexFromItemPath(self, path, column=0):
+        """Model index for the item specified by the given virtual path"""
+        try:
+            item = repotreeitem.findbyitempath(self.rootItem, unicode(path))
+        except ValueError:
+            return QModelIndex()
+        return self._indexFromItem(item, column)
+
     def write(self, fn):
         f = QFile(fn)
         f.open(QIODevice.WriteOnly)
@@ -391,10 +407,10 @@ class RepoTreeModel(QAbstractItemModel):
 
     @pyqtSlot(unicode)
     def _updateShortName(self, uroot):
-        repo = self._repomanager.repoAgent(uroot).rawRepo()
+        repoagent = self._repomanager.repoAgent(uroot)
         it = self.getRepoItem(hglib.fromunicode(uroot))
         if it:
-            it.setShortName(repo.shortname)
+            it.setShortName(repoagent.shortName())
             self._emitItemDataChanged(it)
 
     @pyqtSlot(unicode)
@@ -402,7 +418,7 @@ class RepoTreeModel(QAbstractItemModel):
         repo = self._repomanager.repoAgent(uroot).rawRepo()
         it = self.getRepoItem(hglib.fromunicode(uroot))
         if it:
-            it.setBaseNode(repo[0].node())
+            it.setBaseNode(hglib.repoidnode(repo))
 
     @pyqtSlot(unicode)
     def _updateItem(self, uroot):
