@@ -7,7 +7,7 @@
 # GNU General Public License version 2, incorporated herein by reference.
 
 shortlicense = '''
-Copyright (C) 2008-2013 Steve Borho <steve@borho.org> and others.
+Copyright (C) 2008-2014 Steve Borho <steve@borho.org> and others.
 This is free software; see the source for copying conditions.  There is NO
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 '''
@@ -19,6 +19,7 @@ import subprocess
 
 import mercurial.ui as uimod
 from mercurial import util, fancyopts, cmdutil, extensions, error, scmutil
+from mercurial import pathutil
 
 from tortoisehg.hgqt.i18n import agettext as _
 from tortoisehg.util import hglib, paths, i18n
@@ -32,8 +33,8 @@ except ImportError:
     config_nofork = None
 
 console_commands = 'help thgstatus version'
-nonrepo_commands = '''userconfig shellconfig clone init debugbugreport
-about help version thgstatus serve rejects log'''
+nonrepo_commands = '''userconfig shellconfig clone init debugblockmatcher
+debugbugreport about help version thgstatus serve rejects log'''
 
 def dispatch(args, u=None):
     """run the command specified in args"""
@@ -71,7 +72,11 @@ def portable_fork(ui, opts):
             return
     elif config_nofork:
         return
-    portable_start_fork()
+    try:
+        portable_start_fork()
+    except OSError, inst:
+        ui.warn(_('failed to fork GUI process: %s\n') % inst.strerror)
+        return
     sys.exit(0)
 
 def portable_start_fork(extraargs=None):
@@ -159,7 +164,7 @@ def get_files_from_listfile():
     files = []
     for f in lines:
         try:
-            cpath = scmutil.canonpath(root, cwd, f)
+            cpath = pathutil.canonpath(root, cwd, f)
             # canonpath will abort on .hg/ paths
         except util.Abort:
             continue
@@ -222,6 +227,8 @@ def _parse(ui, args):
 
 def _runcatch(ui, args):
     try:
+        # read --config before doing anything else like Mercurial
+        hglib.parseconfigopts(ui, args)
         try:
             return runcommand(ui, args)
         finally:
@@ -241,11 +248,19 @@ def _runcatch(ui, args):
             help_(ui, 'shortlist')
     except error.RepoError, inst:
         ui.warn(_("abort: %s!\n") % inst)
+    except util.Abort, inst:
+        ui.warn(_("abort: %s\n") % inst)
+        if inst.hint:
+            ui.warn(_("(%s)\n") % inst.hint)
 
     return -1
 
 def runcommand(ui, args):
     cmd, func, args, options, cmdoptions, alias = _parse(ui, args)
+
+    if options['config']:
+        raise util.Abort(_('option --config may not be abbreviated!'))
+
     cmdoptions['alias'] = alias
     ui.setconfig("ui", "verbose", str(bool(options["verbose"])))
     i18n.setlanguage(ui.config('tortoisehg', 'ui.language'))
@@ -369,6 +384,8 @@ globalopts = [
     ('v', 'verbose', None, _('enable additional output')),
     ('q', 'quiet', None, _('suppress output')),
     ('h', 'help', None, _('display help and exit')),
+    ('', 'config', [],
+     _("set/override config option (use 'section.name=value')")),
     ('', 'debugger', None, _('start debugger')),
     ('', 'profile', None, _('print command execution profile')),
     ('', 'nofork', None, _('do not fork GUI process')),
@@ -380,21 +397,8 @@ globalopts = [
 
 # common command functions
 
-def _filelog(ui, repoagent, *pats, **opts):
-    from tortoisehg.hgqt import filedialogs
-    if len(pats) != 1:
-        raise util.Abort(_('requires a single filename'))
-    filename = hglib.canonpaths(pats)[0]
-    return filedialogs.FileLogDialog(repoagent, filename)
-
 def _formatfilerevset(pats):
-    q = []
-    for f in pats:
-        pat = hglib.canonpaths([f])[0]
-        if os.path.isdir(f):
-            q.append('file("%s/**")' % pat)
-        elif os.path.isfile(f):
-            q.append('file("%s")' % pat)
+    q = ["file('path:%s')" % f for f in hglib.canonpaths(pats)]
     return ' or '.join(q)
 
 def _workbench(ui, *pats, **opts):
@@ -439,11 +443,8 @@ def add(ui, repoagent, *pats, **opts):
 def annotate(ui, repoagent, *pats, **opts):
     """annotate dialog"""
     from tortoisehg.hgqt import fileview
-    repo = repoagent.rawRepo()
-    rev = scmutil.revsingle(repo, opts.get('rev')).rev()
-    dlg = _filelog(ui, repoagent, *pats, **opts)
+    dlg = filelog(ui, repoagent, *pats, **opts)
     dlg.setFileViewMode(fileview.AnnMode)
-    dlg.goto(rev)
     if opts.get('line'):
         try:
             lineno = int(opts['line'])
@@ -461,7 +462,7 @@ def archive(ui, repoagent, *pats, **opts):
     """archive dialog"""
     from tortoisehg.hgqt import archive as archivemod
     rev = opts.get('rev')
-    return archivemod.ArchiveDialog(repoagent, rev)
+    return archivemod.createArchiveDialog(repoagent, rev)
 
 @command('^backout',
     [('', 'merge', None, _('merge with old dirstate parent after backout')),
@@ -477,13 +478,18 @@ def backout(ui, repoagent, *pats, **opts):
         rev = pats[0]
     else:
         rev = 'tip'
+    repo = repoagent.rawRepo()
+    rev = scmutil.revsingle(repo, rev).rev()
+    msg = backoutmod.checkrev(repo, rev)
+    if msg:
+        raise util.Abort(hglib.fromunicode(msg))
     return backoutmod.BackoutDialog(repoagent, rev)
 
 @command('^bisect', [], _('thg bisect'))
 def bisect(ui, repoagent, *pats, **opts):
     """bisect dialog"""
     from tortoisehg.hgqt import bisect as bisectmod
-    return bisectmod.BisectDialog(repoagent, opts)
+    return bisectmod.BisectDialog(repoagent)
 
 @command('bookmarks|bookmark',
     [('r', 'rev', '', _('revision'))],
@@ -509,11 +515,11 @@ def bookmark(ui, repoagent, *names, **opts):
      ('', 'pull', None, _('use pull protocol to copy metadata')),
      ('', 'uncompressed', None, _('use uncompressed transfer '
                                   '(fast over LAN)'))],
-    _('thg clone [OPTION]... SOURCE [DEST]'))
+    _('thg clone [OPTION]... [SOURCE] [DEST]'))
 def clone(ui, *pats, **opts):
     """clone tool"""
     from tortoisehg.hgqt import clone as clonemod
-    return clonemod.CloneDialog(pats, opts)
+    return clonemod.CloneDialog(ui, pats, opts)
 
 @command('^commit|ci',
     [('u', 'user', '', _('record user as committer')),
@@ -527,6 +533,12 @@ def commit(ui, repoagent, *pats, **opts):
     os.chdir(repo.root)
     return commitmod.CommitDialog(repoagent, pats, opts)
 
+@command('debugblockmatcher', [], _('thg debugblockmatcher'))
+def debugblockmatcher(ui, *pats, **opts):
+    """show blockmatcher widget"""
+    from tortoisehg.hgqt import blockmatcher
+    return blockmatcher.createTestWidget(ui)
+
 @command('debugbugreport', [], _('thg debugbugreport [TEXT]'))
 def debugbugreport(ui, *pats, **opts):
     """open bugreport dialog by exception"""
@@ -560,6 +572,25 @@ def email(ui, repoagent, *revs, **opts):
     revs = scmutil.revrange(repo, revs)
     return hgemail.EmailDialog(repoagent, revs)
 
+@command('^filelog',
+    [('r', 'rev', '', _('select the specified revision')),
+     ('', 'compare', False, _('side-by-side comparison of revisions'))],
+    _('thg filelog [OPTION]... FILE'))
+def filelog(ui, repoagent, *pats, **opts):
+    """show history of the specified file"""
+    from tortoisehg.hgqt import filedialogs
+    if len(pats) != 1:
+        raise util.Abort(_('requires a single filename'))
+    repo = repoagent.rawRepo()
+    rev = scmutil.revsingle(repo, opts.get('rev')).rev()
+    filename = hglib.canonpaths(pats)[0]
+    if opts.get('compare'):
+        dlg = filedialogs.FileDiffDialog(repoagent, filename)
+    else:
+        dlg = filedialogs.FileLogDialog(repoagent, filename)
+    dlg.goto(rev)
+    return dlg
+
 @command('forget', [], _('thg forget [FILE]...'))
 def forget(ui, repoagent, *pats, **opts):
     """forget selected files"""
@@ -805,7 +836,7 @@ def log(ui, *pats, **opts):
         # TODO: do not instantiate repo here
         repo = thgrepo.repository(ui, root)
         repoagent = repo._pyqtobj
-        return _filelog(ui, repoagent, *pats, **opts)
+        return filelog(ui, repoagent, *pats, **opts)
 
     # Before starting the workbench, we must check if we must try to reuse an
     # existing workbench window (we don't by default)
@@ -841,20 +872,19 @@ def log(ui, *pats, **opts):
     _('thg manifest [-r REV] [FILE]'))
 def manifest(ui, repoagent, *pats, **opts):
     """display the current or given revision of the project manifest"""
-    from tortoisehg.hgqt import manifestdialog
+    from tortoisehg.hgqt import revdetails as revdetailsmod
     repo = repoagent.rawRepo()
     rev = scmutil.revsingle(repo, opts.get('rev')).rev()
-    dlg = manifestdialog.ManifestDialog(repoagent, rev)
+    dlg = revdetailsmod.createManifestDialog(repoagent, rev)
     if pats:
         path = hglib.canonpaths(pats)[0]
+        dlg.setFilePath(hglib.tounicode(path))
         if opts.get('line'):
             try:
                 lineno = int(opts['line'])
             except ValueError:
                 raise util.Abort(_('invalid line number: %s') % opts['line'])
-        else:
-            lineno = None
-        dlg.setSource(hglib.tounicode(path), rev, lineno)
+            dlg.showLine(lineno)
     if opts.get('pattern'):
         dlg.setSearchPattern(hglib.tounicode(opts['pattern']))
     return dlg
@@ -870,6 +900,8 @@ def merge(ui, repoagent, *pats, **opts):
         rev = pats[0]
     if not rev:
         raise util.Abort(_('Merge revision not specified or not found'))
+    repo = repoagent.rawRepo()
+    rev = scmutil.revsingle(repo, rev).rev()
     return mergemod.MergeDialog(repoagent, rev)
 
 @command('postreview',
@@ -928,12 +960,18 @@ def remove(ui, repoagent, *pats, **opts):
     """remove selected files"""
     return quickop.run(ui, repoagent, *pats, **opts)
 
-@command('rename|mv|copy', [], _('thg rename SOURCE [DEST]...'))
-def rename(ui, repoagent, *pats, **opts):
+@command('rename|mv|copy', [], _('thg rename [SOURCE] [DEST]'))
+def rename(ui, repoagent, source=None, dest=None, **opts):
     """rename dialog"""
     from tortoisehg.hgqt import rename as renamemod
+    repo = repoagent.rawRepo()
+    cwd = repo.getcwd()
+    if source:
+        source = hglib.tounicode(pathutil.canonpath(repo.root, cwd, source))
+    if dest:
+        dest = hglib.tounicode(pathutil.canonpath(repo.root, cwd, dest))
     iscopy = (opts.get('alias') == 'copy')
-    return renamemod.RenameDialog(repoagent, pats, iscopy=iscopy)
+    return renamemod.RenameDialog(repoagent, None, source, dest, iscopy)
 
 @command('^repoconfig',
     [('', 'focus', '', _('field to give initial focus'))],
@@ -976,7 +1014,7 @@ def rupdate(ui, repoagent, *pats, **opts):
         rev = opts.get('rev')
     elif len(pats) == 1:
         rev = pats[0]
-    return rupdatemod.rUpdateDialog(repoagent, rev, None, opts)
+    return rupdatemod.createRemoteUpdateDialog(repoagent, rev)
 
 @command('^serve',
     [('', 'web-conf', '', _('name of the hgweb config file (serve more than '
@@ -1046,13 +1084,13 @@ def strip(ui, repoagent, *pats, **opts):
         rev = opts.get('rev')
     elif len(pats) == 1:
         rev = pats[0]
-    return thgstrip.StripDialog(repoagent, rev=rev, opts=opts)
+    return thgstrip.createStripDialog(repoagent, rev=rev, opts=opts)
 
 @command('^sync|synchronize', [], _('thg sync [PEER]'))
 def sync(ui, repoagent, *pats, **opts):
     """synchronize with other repositories"""
     from tortoisehg.hgqt import sync as syncmod
-    w = syncmod.SyncWidget(repoagent, None, **opts)
+    w = syncmod.SyncDialog(repoagent)
     if pats:
         w.setUrl(hglib.tounicode(pats[0]))
     return w
