@@ -12,7 +12,7 @@ from mercurial import ui, util, error, extensions, scmutil, phases
 from tortoisehg.util import hglib, paths, wconfig, i18n, editor
 from tortoisehg.util import terminal, gpg
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, qscilib, thgrepo, customtools
+from tortoisehg.hgqt import qtlib, qscilib, thgrepo, customtools, fileencoding
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -440,10 +440,12 @@ def genEditCombo(opts, defaults=[]):
     opts['defaults'] = defaults
     return SettingsCombo(**opts)
 
-def genIntEditCombo(opts):
+def genIntEditCombo(opts, defaults=None):
     'EditCombo, only allows integer values'
     opts['canedit'] = True
     opts['validator'] = QIntValidator(None)  # missing parent=None on PyQt4.6
+    if defaults:
+        opts['defaults'] = ['%d' % n for n in defaults]
     return SettingsCombo(**opts)
 
 def genLineEditBox(opts):
@@ -527,11 +529,11 @@ def genCheckBox(opts):
 class _fi(object):
     """Information of each field"""
     __slots__ = ('label', 'cpath', 'values', 'tooltip',
-                 'restartneeded', 'globalonly',
+                 'restartneeded', 'globalonly', 'noglobal',
                  'master', 'visible')
 
     def __init__(self, label, cpath, values, tooltip,
-                 restartneeded=False, globalonly=False,
+                 restartneeded=False, globalonly=False, noglobal=False,
                  master=None, visible=None):
         self.label = label
         self.cpath = cpath
@@ -539,6 +541,7 @@ class _fi(object):
         self.tooltip = tooltip
         self.restartneeded = restartneeded
         self.globalonly = globalonly
+        self.noglobal = noglobal
         self.master = master
         self.visible = visible
 
@@ -594,18 +597,20 @@ INFO = (
           'TortoiseHg windows. '
           'Default: 8')),
     _fi(_('Force Repo Tab'), 'tortoisehg.forcerepotab', genBoolRBGroup,
-        _('Always show repo tabs, even for a single repo. Default: False')),
+        _('Always show repo tabs, even for a single repo. Default: False'),
+        globalonly=True),
     _fi(_('Monitor Repo Changes'), 'tortoisehg.monitorrepo',
         (genDefaultCombo, ['always', 'localonly', 'never']),
         _('Specify the target filesystem where TortoiseHg monitors changes. '
-          'Default: always')),
+          'Default: localonly')),
     _fi(_('Max Diff Size'), 'tortoisehg.maxdiff', genIntEditCombo,
         _('The maximum size file (in KB) that TortoiseHg will '
           'show changes for in the changelog, status, and commit windows. '
           'A value of zero implies no limit.  Default: 1024 (1MB)')),
     _fi(_('Fork GUI'), 'tortoisehg.guifork', genBoolRBGroup,
         _('When running from the command line, fork a background '
-          'process to run graphical dialogs.  Default: True')),
+          'process to run graphical dialogs.  Default: True'),
+        globalonly=True),
     _fi(_('Full Path Title'), 'tortoisehg.fullpath', genBoolRBGroup,
         _('Show a full directory path of the repository in the dialog title '
           'instead of just the root directory name.  Default: False')),
@@ -616,7 +621,7 @@ INFO = (
           'with changes on both sides of the merge will report as conflicting, '
           'even if the edits are to different parts of the file. In either '
           'case, when conflicts occur, the user will be invited to review and '
-          'resolve changes manually. Default: False.')),
+          'resolve changes manually. Default: True.')),
     )),
 
 ({'name': 'log', 'label': _('Workbench'), 'icon': 'menulog'}, (
@@ -628,7 +633,7 @@ INFO = (
           'context menu. Default: True'),
         restartneeded=True, globalonly=True),
     _fi(_('Default widget'), 'tortoisehg.defaultwidget', (genDefaultCombo,
-        ['revdetails', 'commit', 'sync', 'manifest', 'search']),
+        ['revdetails', 'commit', 'sync', 'search']),
         _('Select the initial widget that will be shown when opening a '
           'repository. '
           'Default: revdetails')),
@@ -643,11 +648,10 @@ INFO = (
         'tortoisehg.opentabsaftercurrent', genBoolRBGroup,
         _('Should new tabs be open next to the current tab? '
           'If False new tabs will be open after the last tab. '
-          'Default: True')),
+          'Default: True'),
+        globalonly=True),
     _fi(_('Author Coloring'), 'tortoisehg.authorcolor', genBoolRBGroup,
-        _('Color changesets by author name.  If not enabled, '
-          'the changes are colored green for merge, red for '
-          'non-trivial parents, black for normal. '
+        _('Color changesets by author name. '
           'Default: False')),
     _fi(_('Full Authorname'), 'tortoisehg.fullauthorname', genBoolRBGroup,
         _('Show full authorname in Logview. If not enabled, '
@@ -663,12 +667,12 @@ INFO = (
         _('Specify which task buttons you want to show on the task toolbar '
           'and in which order.<br>Type a list of the task button names. '
           'Add separators by putting "|" between task button names.<br>'
-          'Valid names are: log commit mq sync manifest grep and pbranch.<br>'
-          'Default: log commit manifest grep pbranch | sync'),
+          'Valid names are: log commit sync grep and pbranch.<br>'
+          'Default: log commit grep pbranch | sync'),
         restartneeded=True, globalonly=True),
     _fi(_('Long Summary'), 'tortoisehg.longsummary', genBoolRBGroup,
         _('If true, concatenate multiple lines of changeset summary '
-          'until they reach 80 characters. '
+          'and truncate them at 80 characters as necessary. '
           'Default: False')),
     _fi(_('Log Batch Size'), 'tortoisehg.graphlimit', genIntEditCombo,
         _('The number of revisions to read and display in the '
@@ -703,6 +707,12 @@ INFO = (
           '<li><b>never</b>: Never show any prompt to activate any bookmarks.'
           '</ul><p>'
           'Default: prompt')),
+    _fi(_('Show Family Line'), 'tortoisehg.showfamilyline', genBoolRBGroup,
+        _('Show indirect revision dependency on the revision graph '
+          'when filtered by revset. Default: True<p>'
+          '<b>Note</b>: Calculating family line may be slow in some cases. '
+          'This option is expected to be removed if the performance issue is '
+          'solved.')),
     )),
 ({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'menucommit'},
  (
@@ -739,6 +749,13 @@ INFO = (
     _fi(_('Secret MQ Patches'), 'mq.secret', genBoolRBGroup,
         _('Make MQ patches secret (instead of draft). '
           'Default: False')),
+    _fi(_('Check Subrepo Phase'), 'phases.checksubrepos',
+        (genDefaultCombo, ['ignore', 'follow', 'abort']),
+        _('Check the phase of the current revision of each subrepository.  '
+          'For settings other than "ignore", the phase of the current '
+          'revision of each subrepository is checked before committing the '
+          'parent repository.  '
+          'Default: follow')),
     _fi(_('Monitor working<br>directory changes'),
         'tortoisehg.refreshwdstatus',
         (genDefaultCombo, ['auto', 'always', 'alwayslocal']),
@@ -807,16 +824,21 @@ INFO = (
     )),
 
 ({'name': 'web', 'label': _('Server'), 'icon': 'proxy'}, (
-    _fi(_('<b>Behavior:</b>'), None, genSpacer, ''),
+    _fi(_('<b>Repository Details:</b>'), None, genSpacer, ''),
+    _fi(_('Name'), 'web.name', genEditCombo,
+        _('Repository name to use in the web interface, and by TortoiseHg '
+          'as a shorthand name.  Default is the working directory.'),
+        noglobal=True),
+    _fi(_('Encoding'), 'web.encoding',
+        (genEditCombo, fileencoding.knownencodings()),
+        _('Character encoding of files in the repository, used by the web '
+          'interface and TortoiseHg.')),
     _fi(_("'Publishing' repository"), 'phases.publish', genBoolRBGroup,
         _('Controls draft phase behavior when working as a server. When true, '
           'pushed changesets are set to public in both client and server and '
           'pulled or cloned changesets are set to public in the client. '
           'Default: True')),
     _fi(_('<b>Web Server:</b>'), None, genSpacer, ''),
-    _fi(_('Name'), 'web.name', genEditCombo,
-        _('Repository name to use in the web interface, and by TortoiseHg '
-          'as a shorthand name.  Default is the working directory.')),
     _fi(_('Description'), 'web.description', genEditCombo,
         _("Textual description of the repository's purpose or "
           'contents.')),
@@ -857,8 +879,6 @@ INFO = (
           'denied, and any authenticated user name present in this list '
           '(separated by whitespace or ",") is also denied. The contents '
           'of the deny_push list are examined before the allow_push list.')),
-    _fi(_('Encoding'), 'web.encoding', (genEditCombo, ['UTF-8']),
-        _('Character encoding name')),
     )),
 
 ({'name': 'proxy', 'label': _('Proxy'), 'icon': QStyle.SP_DriveNetIcon}, (
@@ -894,12 +914,13 @@ INFO = (
           'is enough to use sendmail to send messages.')),
     _fi(_('SMTP Host'), 'smtp.host', genEditCombo,
         _('Host name of mail server')),
-    _fi(_('SMTP Port'), 'smtp.port', genIntEditCombo,
+    _fi(_('SMTP Port'), 'smtp.port', (genIntEditCombo, [25, 465, 587]),
         _('Port to connect to on mail server. '
           'Default: 25')),
-    _fi(_('SMTP TLS'), 'smtp.tls', genBoolRBGroup,
-        _('Connect to mail server using TLS. '
-          'Default: False')),
+    _fi(_('SMTP TLS'), 'smtp.tls',
+        (genDefaultCombo, ['starttls', 'smtps', 'none']),
+        _('Method to enable TLS when connecting to mail server. '
+          'Default: none')),
     _fi(_('SMTP Username'), 'smtp.username', genEditCombo,
         _('Username to authenticate to mail server with')),
     _fi(_('SMTP Password'), 'smtp.password', genPasswordEntry,
@@ -1191,6 +1212,7 @@ class SettingsDialog(QDialog):
                                   _('no repo at ') + uroot, parent=self)
 
         if repo:
+            repoagent = repo._pyqtobj  # TODO
             if 'projrc' in repo.extensions():
                 projrcpath = os.sep.join([repo.root, '.hg', 'projrc'])
                 if os.path.exists(projrcpath):
@@ -1198,13 +1220,14 @@ class SettingsDialog(QDialog):
                                         readonly=True)
                     self.conftabs.addTab(rtab, qtlib.geticon('settings_projrc'),
                                          _('%s project settings (.hg/projrc)')
-                                         % repo.shortname)
+                                         % repoagent.shortName())
                     rtab.restartRequested.connect(self._pushRestartRequest)
 
             reporcpath = os.sep.join([repo.root, '.hg', 'hgrc'])
             rtab = SettingsForm(rcpath=reporcpath, focus=focus)
             self.conftabs.addTab(rtab, qtlib.geticon('settings_repo'),
-                                 _('%s repository settings') % repo.shortname)
+                                 _('%s repository settings')
+                                 % repoagent.shortName())
             rtab.restartRequested.connect(self._pushRestartRequest)
 
         BB = QDialogButtonBox
@@ -1504,6 +1527,8 @@ class SettingsForm(QWidget):
                 w = func(opts)
             if e.globalonly:
                 w.setEnabled(self.rcpath == scmutil.userrcpath())
+            elif e.noglobal:
+                w.setEnabled(self.rcpath != scmutil.userrcpath())
             lbl = QLabel(e.label)
             lbl.setToolTip(e.tooltip)
             widgets.append(w)
