@@ -29,7 +29,6 @@ class ShelveDialog(QDialog):
         self.setWindowIcon(qtlib.geticon('shelve'))
 
         self._repoagent = repoagent
-        repo = repoagent.rawRepo()
         self.shelves = []
         self.patches = []
 
@@ -73,7 +72,7 @@ class ShelveDialog(QDialog):
         ahbox.addWidget(self.clearShelfButtonA)
         ahbox.addWidget(self.delShelfButtonA)
 
-        self.browsea = chunks.ChunksWidget(self._repoagent, self, True)
+        self.browsea = chunks.ChunksWidget(self._repoagent, self)
         self.browsea.splitter.splitterMoved.connect(self.linkSplitters)
         self.browsea.linkActivated.connect(self.linkActivated)
         self.browsea.showMessage.connect(self.showMessage)
@@ -103,17 +102,18 @@ class ShelveDialog(QDialog):
         bhbox.addWidget(self.clearShelfButtonB)
         bhbox.addWidget(self.delShelfButtonB)
 
-        self.browseb = chunks.ChunksWidget(self._repoagent, self, True)
+        self.browseb = chunks.ChunksWidget(self._repoagent, self)
         self.browseb.splitter.splitterMoved.connect(self.linkSplitters)
         self.browseb.linkActivated.connect(self.linkActivated)
         self.browseb.showMessage.connect(self.showMessage)
         bvbox.addWidget(self.browseb)
 
         self.lefttbar = QToolBar(_('Left Toolbar'), objectName='lefttbar')
+        self.lefttbar.setIconSize(qtlib.toolBarIconSize())
         self.lefttbar.setStyleSheet(qtlib.tbstylesheet)
         self.tbarhbox.addWidget(self.lefttbar)
         self.deletea = a = QAction(_('Delete selected chunks'), self)
-        self.deletea.triggered.connect(self.browsea.deleteSelectedChunks)
+        self.deletea.triggered.connect(self.deleteChunksA)
         a.setIcon(qtlib.geticon('thg-shelve-delete-left'))
         self.lefttbar.addAction(self.deletea)
         self.allright = a = QAction(_('Move all files right'), self)
@@ -133,6 +133,7 @@ class ShelveDialog(QDialog):
         self.lefttbar.addAction(self.chunksright)
 
         self.rbar = QToolBar(_('Refresh Toolbar'), objectName='rbar')
+        self.rbar.setIconSize(qtlib.toolBarIconSize())
         self.rbar.setStyleSheet(qtlib.tbstylesheet)
         self.tbarhbox.addStretch(1)
         self.tbarhbox.addWidget(self.rbar)
@@ -147,6 +148,7 @@ class ShelveDialog(QDialog):
         self.rbar.addAction(self.actionNew)
 
         self.righttbar = QToolBar(_('Right Toolbar'), objectName='righttbar')
+        self.righttbar.setIconSize(qtlib.toolBarIconSize())
         self.righttbar.setStyleSheet(qtlib.tbstylesheet)
         self.tbarhbox.addStretch(1)
         self.tbarhbox.addWidget(self.righttbar)
@@ -166,7 +168,7 @@ class ShelveDialog(QDialog):
         a.setIcon(qtlib.geticon('thg-shelve-move-left-all'))
         self.righttbar.addAction(self.allleft)
         self.deleteb = a = QAction(_('Delete selected chunks'), self)
-        self.deleteb.triggered.connect(self.browseb.deleteSelectedChunks)
+        self.deleteb.triggered.connect(self.deleteChunksB)
         a.setIcon(qtlib.geticon('thg-shelve-delete-right'))
         self.righttbar.addAction(self.deleteb)
 
@@ -194,7 +196,8 @@ class ShelveDialog(QDialog):
         self.refreshCombos()
         repoagent.repositoryChanged.connect(self.refreshCombos)
 
-        self.setWindowTitle(_('TortoiseHg Shelve - %s') % repo.displayname)
+        self.setWindowTitle(_('TortoiseHg Shelve - %s')
+                            % repoagent.displayName())
         self.restoreSettings()
 
     @property
@@ -247,6 +250,23 @@ class ShelveDialog(QDialog):
         if self.browsea.mergeChunks(file, chunks):
             self.browseb.deleteSelectedChunks()
 
+    @pyqtSlot()
+    def deleteChunksA(self):
+        if self.comboa.currentIndex() == 0:
+            msg = _('Delete selected chunks from working copy?')
+        else:
+            f = hglib.tounicode(os.path.basename(self.currentPatchA()))
+            msg = _('Delete selected chunks from shelf file %s?') % f
+        if qtlib.QuestionMsgBox(_('Are you sure?'), msg, parent=self):
+            self.browsea.deleteSelectedChunks()
+
+    @pyqtSlot()
+    def deleteChunksB(self):
+        f = hglib.tounicode(os.path.basename(self.currentPatchB()))
+        msg = _('Delete selected chunks from shelf file %s?') % f
+        if qtlib.QuestionMsgBox(_('Are you sure?'), msg, parent=self):
+            self.browseb.deleteSelectedChunks()
+
     @pyqtSlot()
     def newShelfPressed(self):
         self.newShelf(True)
@@ -387,7 +407,7 @@ class ShelveDialog(QDialog):
     def refreshCombos(self):
         shelvea, shelveb = self.currentPatchA(), self.currentPatchB()
 
-        # Note that thgshelves returns the shelve list ordered from newest to 
+        # Note that thgshelves returns the shelve list ordered from newest to
         # oldest
         shelves = self.repo.thgshelves()
         disp = [_('Shelf: %s') % hglib.tounicode(s) for s in shelves]
