@@ -11,44 +11,30 @@ from mercurial import error
 
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import cmdui, csinfo, qtlib, thgrepo, resolve
+from tortoisehg.hgqt import cmdcore, cmdui, csinfo, qtlib, resolve
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-class UpdateDialog(QDialog):
-
-    output = pyqtSignal(QString, QString)
-    progress = pyqtSignal(QString, object, QString, QString, object)
-    makeLogVisible = pyqtSignal(bool)
+class UpdateWidget(cmdui.AbstractCmdWidget):
 
     def __init__(self, repoagent, rev=None, parent=None, opts={}):
-        super(UpdateDialog, self).__init__(parent)
-        self.setWindowFlags(self.windowFlags() & \
-                            ~Qt.WindowContextHelpButtonHint)
-
+        super(UpdateWidget, self).__init__(parent)
+        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self._repoagent = repoagent
         repo = repoagent.rawRepo()
 
-        # base layout box
-        box = QVBoxLayout()
-        box.setSpacing(6)
-
-        ## main layout grid
-        self.grid = QGridLayout()
-        self.grid.setSpacing(6)
-        box.addLayout(self.grid)
+        ## main layout
+        form = QFormLayout()
+        form.setContentsMargins(0, 0, 0, 0)
+        form.setSpacing(6)
+        self.setLayout(form)
 
         ### target revision combo
         self.rev_combo = combo = QComboBox()
         combo.setEditable(True)
-        self.grid.addWidget(QLabel(_('Update to:')), 0, 0)
-        self.grid.addWidget(combo, 0, 1)
-
-        # Give the combo box a minimum width that will ensure that the dialog is
-        # large enough to fit the additional progress bar that will appear when
-        # updating subrepositories.
-        combo.setMinimumWidth(450)
+        combo.installEventFilter(qtlib.BadCompletionBlocker(combo))
+        form.addRow(_('Update to:'), combo)
 
         # always include integer revision
         try:
@@ -59,13 +45,10 @@ class UpdateDialog(QDialog):
         except error.RepoLookupError:
             pass
 
-        for name in repo.namedbranches:
-            combo.addItem(hglib.tounicode(name))
-
+        combo.addItems(map(hglib.tounicode, hglib.namedbranches(repo)))
         tags = list(self.repo.tags()) + repo._bookmarks.keys()
         tags.sort(reverse=True)
-        for tag in tags:
-            combo.addItem(hglib.tounicode(tag))
+        combo.addItems(map(hglib.tounicode, tags))
 
         if rev is None:
             selecturev = hglib.tounicode(self.repo.dirstate.branch())
@@ -82,132 +65,95 @@ class UpdateDialog(QDialog):
         style = csinfo.labelstyle(contents=items, width=350, selectable=True)
         factory = csinfo.factory(self.repo, style=style)
         self.target_info = factory()
-        self.grid.addWidget(QLabel(_('Target:')), 1, 0, Qt.AlignLeft | Qt.AlignTop)
-        self.grid.addWidget(self.target_info, 1, 1)
+        form.addRow(_('Target:'), self.target_info)
 
         ### parent revision info
         self.ctxs = self.repo[None].parents()
         if len(self.ctxs) == 2:
             self.p1_info = factory()
-            self.grid.addWidget(QLabel(_('Parent 1:')), 2, 0, Qt.AlignLeft | Qt.AlignTop)
-            self.grid.addWidget(self.p1_info, 2, 1)
+            form.addRow(_('Parent 1:'), self.p1_info)
             self.p2_info = factory()
-            self.grid.addWidget(QLabel(_('Parent 2:')), 3, 0, Qt.AlignLeft | Qt.AlignTop)
-            self.grid.addWidget(self.p2_info, 3, 1)
+            form.addRow(_('Parent 2:'), self.p2_info)
         else:
             self.p1_info = factory()
-            self.grid.addWidget(QLabel(_('Parent:')), 2, 0, Qt.AlignLeft | Qt.AlignTop)
-            self.grid.addWidget(self.p1_info, 2, 1)
+            form.addRow(_('Parent:'), self.p1_info)
+
+        # show a subrepo "pull path" combo, with the
+        # default path as the first (and default) path
+        self.path_combo_label = QLabel(_('Pull subrepos from:'))
+        self.path_combo = QComboBox(self)
+        syncpaths = dict(repo.ui.configitems('paths'))
+        aliases = sorted(syncpaths)
+        # make sure that the default path is the first one
+        if 'default' in aliases:
+            aliases.remove('default')
+            aliases.insert(0, 'default')
+        for n, alias in enumerate(aliases):
+            self.path_combo.addItem(hglib.tounicode(alias))
+            self.path_combo.setItemData(
+                n, hglib.tounicode(syncpaths[alias]))
+        self.path_combo.currentIndexChanged.connect(
+            self._updatePathComboTooltip)
+        self._updatePathComboTooltip(0)
+        form.addRow(self.path_combo_label, self.path_combo)
 
         ### options
         self.optbox = QVBoxLayout()
         self.optbox.setSpacing(6)
-        expander = qtlib.ExpanderLabel(_('Options:'), False)
+        self.optexpander = expander = qtlib.ExpanderLabel(_('Options:'), False)
         expander.expanded.connect(self.show_options)
-        row = self.grid.rowCount()
-        self.grid.addWidget(expander, row, 0, Qt.AlignLeft | Qt.AlignTop)
-        self.grid.addLayout(self.optbox, row, 1)
+        form.addRow(expander, self.optbox)
 
         self.verbose_chk = QCheckBox(_('List updated files (--verbose)'))
         self.discard_chk = QCheckBox(_('Discard local changes, no backup '
                                        '(-C/--clean)'))
         self.merge_chk = QCheckBox(_('Always merge (when possible)'))
-        self.autoresolve_chk = QCheckBox(_('Automatically resolve merge conflicts '
-                                           'where possible'))
-        self.showlog_chk = QCheckBox(_('Always show command log'))
+        self.autoresolve_chk = QCheckBox(_('Automatically resolve merge '
+                                           'conflicts where possible'))
         self.optbox.addWidget(self.verbose_chk)
         self.optbox.addWidget(self.discard_chk)
         self.optbox.addWidget(self.merge_chk)
         self.optbox.addWidget(self.autoresolve_chk)
-        self.optbox.addWidget(self.showlog_chk)
-
-        s = QSettings()
 
         self.discard_chk.setChecked(bool(opts.get('clean')))
 
-        #### Persisted Options
-        self.merge_chk.setChecked(
-            QSettings().value('update/merge', False).toBool())
-
-        self.autoresolve_chk.setChecked(
-            repo.ui.configbool('tortoisehg', 'autoresolve', False) or
-                s.value('update/autoresolve', False).toBool())
-
-        self.showlog_chk.setChecked(s.value('update/showlog', False).toBool())
-        self.verbose_chk.setChecked(s.value('update/verbose', False).toBool())
-
-        ## command widget
-        self.cmd = cmdui.Widget(True, True, self)
-        self.cmd.commandStarted.connect(self.command_started)
-        self.cmd.commandFinished.connect(self.command_finished)
-        self.cmd.commandCanceling.connect(self.command_canceling)
-        self.cmd.output.connect(self.output)
-        self.cmd.makeLogVisible.connect(self.makeLogVisible)
-        self.cmd.progress.connect(self.progress)
-        box.addWidget(self.cmd)
-
-        ## bottom buttons
-        buttons = QDialogButtonBox()
-        self.cancel_btn = buttons.addButton(QDialogButtonBox.Cancel)
-        self.cancel_btn.clicked.connect(self.cancel_clicked)
-        self.close_btn = buttons.addButton(QDialogButtonBox.Close)
-        self.close_btn.clicked.connect(self.reject)
-        self.close_btn.setAutoDefault(False)
-        self.update_btn = buttons.addButton(_('&Update'),
-                                            QDialogButtonBox.ActionRole)
-        self.update_btn.clicked.connect(self.update)
-        self.detail_btn = buttons.addButton(_('Detail'),
-                                            QDialogButtonBox.ResetRole)
-        self.detail_btn.setAutoDefault(False)
-        self.detail_btn.setCheckable(True)
-        self.detail_btn.toggled.connect(self.detail_toggled)
-        box.addWidget(buttons)
-
         # signal handlers
         self.rev_combo.editTextChanged.connect(self.update_info)
         self.discard_chk.toggled.connect(self.update_info)
 
-        # dialog setting
-        self.setLayout(box)
-        self.layout().setSizeConstraint(QLayout.SetFixedSize)
-        self.setWindowTitle(_('Update - %s') % self.repo.displayname)
-        self.setWindowIcon(qtlib.geticon('hg-update'))
-
         # prepare to show
-        self.cmd.setHidden(True)
-        self.cancel_btn.setHidden(True)
-        self.detail_btn.setHidden(True)
         self.merge_chk.setHidden(True)
         self.autoresolve_chk.setHidden(True)
-        self.showlog_chk.setHidden(True)
         self.update_info()
-        if not self.update_btn.isEnabled():
-            self.rev_combo.lineEdit().selectAll()  # need to change rev
+        if not self.canRunCommand():
+            # need to change rev
+            self.rev_combo.lineEdit().selectAll()
+
+    def readSettings(self, qs):
+        self.merge_chk.setChecked(qs.value('merge').toBool())
+        self.autoresolve_chk.setChecked(
+            self.repo.ui.configbool('tortoisehg', 'autoresolve',
+                                    qs.value('autoresolve', True).toBool()))
+        self.verbose_chk.setChecked(qs.value('verbose').toBool())
 
         # expand options if a hidden one is checked
-        hiddenOptionsChecked = self.hiddenSettingIsChecked()
-        self.show_options(hiddenOptionsChecked)
-        expander.set_expanded(hiddenOptionsChecked)
+        self.optexpander.set_expanded(self.hiddenSettingIsChecked())
 
-    ### Private Methods ###
+    def writeSettings(self, qs):
+        qs.setValue('merge', self.merge_chk.isChecked())
+        qs.setValue('autoresolve', self.autoresolve_chk.isChecked())
+        qs.setValue('verbose', self.verbose_chk.isChecked())
 
     @property
     def repo(self):
         return self._repoagent.rawRepo()
 
     def hiddenSettingIsChecked(self):
-        if self.merge_chk.isChecked() or self.autoresolve_chk.isChecked() or self.showlog_chk.isChecked():
-            return True
-        else:
-            return False
+        return (self.merge_chk.isChecked()
+                or self.autoresolve_chk.isChecked())
 
-    def saveSettings(self):
-        QSettings().setValue('update/verbose', self.verbose_chk.isChecked())
-        QSettings().setValue('update/merge', self.merge_chk.isChecked())
-        QSettings().setValue('update/autoresolve', self.autoresolve_chk.isChecked())
-        QSettings().setValue('update/showlog', self.showlog_chk.isChecked())
-
-    def update_info(self, *args):
+    @pyqtSlot()
+    def update_info(self):
         self.p1_info.update(self.ctxs[0].node())
         merge = len(self.ctxs) == 2
         if merge:
@@ -215,25 +161,38 @@ class UpdateDialog(QDialog):
         new_rev = hglib.fromunicode(self.rev_combo.currentText())
         if new_rev == 'null':
             self.target_info.setText(_('remove working directory'))
-            self.update_btn.setEnabled(True)
+            self.commandChanged.emit()
             return
         try:
             new_ctx = self.repo[new_rev]
             if not merge and new_ctx.rev() == self.ctxs[0].rev() \
                     and not new_ctx.bookmarks():
                 self.target_info.setText(_('(same as parent)'))
-                clean = self.discard_chk.isChecked()
-                self.update_btn.setEnabled(clean)
             else:
                 self.target_info.update(self.repo[new_rev])
-                self.update_btn.setEnabled(True)
+            # only show the path combo when there are multiple paths
+            # and the target revision has subrepos
+            showpathcombo = self.path_combo.count() > 1 and \
+                '.hgsubstate' in new_ctx
+            self.path_combo_label.setVisible(showpathcombo)
+            self.path_combo.setVisible(showpathcombo)
         except (error.LookupError, error.RepoLookupError, error.RepoError):
             self.target_info.setText(_('unknown revision!'))
-            self.update_btn.setDisabled(True)
+        self.commandChanged.emit()
 
-    def update(self):
-        self.saveSettings()
-        cmdline = ['update', '--repository', self.repo.root]
+    def canRunCommand(self):
+        new_rev = hglib.fromunicode(self.rev_combo.currentText())
+        try:
+            new_ctx = self.repo[new_rev]
+        except (error.LookupError, error.RepoLookupError, error.RepoError):
+            return False
+        return (self.discard_chk.isChecked()
+                or len(self.ctxs) == 2
+                or new_ctx.rev() != self.ctxs[0].rev()
+                or bool(new_ctx.bookmarks()))
+
+    def runCommand(self):
+        cmdline = ['update']
         if self.verbose_chk.isChecked():
             cmdline += ['--verbose']
         cmdline += ['--config', 'ui.merge=internal:' +
@@ -258,10 +217,12 @@ class UpdateDialog(QDialog):
                         activatebookmark = qtlib.QuestionMsgBox(
                             _('Activate bookmark?'),
                             _('The selected revision (%s) has a bookmark on it '
-                            'called "<i>%s</i>".<p>Do you want to activate it?'
-                            '<br></b><i>You can disable this prompt by configuring '
-                            'Settings/Workbench/Activate Bookmarks</i>') \
-                            % (hglib.tounicode(rev), bookmarks[0]))
+                              'called "<i>%s</i>".<p>Do you want to activate '
+                              'it?<br></b>'
+                              '<i>You can disable this prompt by configuring '
+                              'Settings/Workbench/Activate Bookmarks</i>') \
+                            % (hglib.tounicode(rev),
+                               hglib.tounicode(bookmarks[0])))
                     if activatebookmark:
                         selectedbookmark = bookmarks[0]
                 else:
@@ -270,11 +231,12 @@ class UpdateDialog(QDialog):
                     selectedbookmark = qtlib.ChoicePrompt(
                         _('Activate bookmark?'),
                         _('The selected revision (<i>%s</i>) has <i>%d</i> '
-                        'bookmarks on it.<p>Select the bookmark that you want '
-                        'to activate and click <i>OK</i>.<p>Click <i>Cancel</i> '
-                        'if you don\'t want to activate any of them.<p>'
-                        '<p><i>You can disable this prompt by configuring '
-                        'Settings/Workbench/Activate Bookmarks</i><p>') \
+                          'bookmarks on it.<p>Select the bookmark that you '
+                          'want to activate and click <i>OK</i>.'
+                          "<p>Click <i>Cancel</i> if you don't want to "
+                          'activate any of them.<p><p>'
+                          '<i>You can disable this prompt by configuring '
+                          'Settings/Workbench/Activate Bookmarks</i><p>') \
                         % (hglib.tounicode(rev), len(bookmarks)),
                         self, bookmarks, self.repo._bookmarkcurrent).run()
                 if selectedbookmark:
@@ -283,19 +245,32 @@ class UpdateDialog(QDialog):
                     deactivatebookmark = qtlib.QuestionMsgBox(
                         _('Deactivate current bookmark?'),
                         _('Do you really want to deactivate the <i>%s</i> '
-                        'bookmark?') % self.repo._bookmarkcurrent)
+                          'bookmark?')
+                        % hglib.tounicode(self.repo._bookmarkcurrent))
                     if deactivatebookmark:
-                        cmdline = ['bookmark', '--repository', self.repo.root]
+                        cmdline = ['bookmark']
                         if self.verbose_chk.isChecked():
                             cmdline += ['--verbose']
-                        cmdline += ['-i', self.repo._bookmarkcurrent]
-                        self.repo.incrementBusyCount()
-                        self.cmd.run(cmdline)
-                    return
+                        cmdline += ['-i',
+                                    hglib.tounicode(self.repo._bookmarkcurrent)]
+                        return self._repoagent.runCommand(cmdline, self)
+                    return cmdcore.nullCmdSession()
 
         cmdline.append('--rev')
         cmdline.append(rev)
 
+        pullpathname = hglib.fromunicode(
+            self.path_combo.currentText())
+        if pullpathname and pullpathname != 'default':
+            # We must tell mercurial to pull any missing repository
+            # revisions from the selected path. The only way to do so is
+            # to temporarily set the default path to the selected path URL
+            pullpath = hglib.fromunicode(
+                self.path_combo.itemData(
+                    self.path_combo.currentIndex()).toString())
+            cmdline.append('--config')
+            cmdline.append('paths.default=%s' % pullpath)
+
         if self.discard_chk.isChecked():
             cmdline.append('--clean')
         else:
@@ -303,7 +278,7 @@ class UpdateDialog(QDialog):
             try:
                 node = self.repo.hgchangectx(rev)
             except (error.LookupError, error.RepoLookupError, error.RepoError):
-                return
+                return cmdcore.nullCmdSession()
             def isclean():
                 '''whether WD is changed'''
                 try:
@@ -332,11 +307,13 @@ class UpdateDialog(QDialog):
                 msg = _('Detected uncommitted local changes in working tree.\n'
                         'Please select to continue:\n')
                 data = {'discard': (_('&Discard'),
-                                    _('Discard - discard local changes, no backup')),
+                                    _('Discard - discard local changes, no '
+                                      'backup')),
                         'shelve': (_('&Shelve'),
                                   _('Shelve - move local changes to a patch')),
                         'merge': (_('&Merge'),
-                                  _('Merge - allow to merge with local changes')),}
+                                  _('Merge - allow to merge with local '
+                                    'changes'))}
 
                 opts = ['discard']
                 if not ismergedchange():
@@ -374,60 +351,45 @@ class UpdateDialog(QDialog):
                     dlg = shelve.ShelveDialog(self._repoagent, self)
                     dlg.finished.connect(dlg.deleteLater)
                     dlg.exec_()
-                    return
+                    return cmdcore.nullCmdSession()
                 elif clicked == 'merge':
                     pass # no args
                 else:
-                    return
+                    return cmdcore.nullCmdSession()
 
-        # start updating
-        self.repo.incrementBusyCount()
-        self.cmd.run(cmdline)
-
-    ### Signal Handlers ###
-
-    def cancel_clicked(self):
-        self.cmd.cancel()
-        self.reject()
-
-    def detail_toggled(self, checked):
-        self.cmd.setShowOutput(checked)
+        cmdline = map(hglib.tounicode, cmdline)
+        return self._repoagent.runCommand(cmdline, self)
 
+    @pyqtSlot(bool)
     def show_options(self, visible):
         self.merge_chk.setShown(visible)
         self.autoresolve_chk.setShown(visible)
-        self.showlog_chk.setShown(visible)
 
-    def command_started(self):
-        self.cmd.setShown(True)
-        if self.showlog_chk.isChecked():
-            self.detail_btn.setChecked(True)
-        self.update_btn.setHidden(True)
-        self.close_btn.setHidden(True)
-        self.cancel_btn.setShown(True)
-        self.detail_btn.setShown(True)
+    @pyqtSlot(int)
+    def _updatePathComboTooltip(self, idx):
+        self.path_combo.setToolTip(self.path_combo.itemData(idx).toString())
 
-    def command_finished(self, ret):
-        self.repo.decrementBusyCount()
-        if ret not in (0, 1) or self.cmd.outputShown():
-            self.detail_btn.setChecked(True)
-            self.close_btn.setShown(True)
-            self.close_btn.setAutoDefault(True)
-            self.close_btn.setFocus()
-            self.cancel_btn.setHidden(True)
-        else:
-            self.accept()
 
-    def accept(self):
-        for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if status == 'u':
-                qtlib.InfoMsgBox(_('Merge caused file conflicts'),
-                                 _('File conflicts need to be resolved'))
-                dlg = resolve.ResolveDialog(self._repoagent, self)
-                dlg.finished.connect(dlg.deleteLater)
-                dlg.exec_()
-                break
-        super(UpdateDialog, self).accept()
+class UpdateDialog(cmdui.CmdControlDialog):
 
-    def command_canceling(self):
-        self.cancel_btn.setDisabled(True)
+    def __init__(self, repoagent, rev=None, parent=None, opts={}):
+        super(UpdateDialog, self).__init__(parent)
+        self._repoagent = repoagent
+
+        self.setWindowTitle(_('Update - %s') % repoagent.displayName())
+        self.setWindowIcon(qtlib.geticon('hg-update'))
+        self.setObjectName('update')
+        self.setRunButtonText(_('&Update'))
+        self.setCommandWidget(UpdateWidget(repoagent, rev, self, opts))
+        self.commandFinished.connect(self._checkMergeConflicts)
+
+    @pyqtSlot(int)
+    def _checkMergeConflicts(self, ret):
+        if ret != 1:
+            return
+        qtlib.InfoMsgBox(_('Merge caused file conflicts'),
+                         _('File conflicts need to be resolved'))
+        dlg = resolve.ResolveDialog(self._repoagent, self)
+        dlg.exec_()
+        if not self.isLogVisible():
+            self.reject()
