@@ -29,7 +29,11 @@ class patchctx(object):
             The handle is NOT closed.
         """
         self._path = patchpath
-        self._patchname = os.path.basename(patchpath)
+        if rev:
+            assert isinstance(rev, str)
+            self._patchname = rev
+        else:
+            self._patchname = os.path.basename(patchpath)
         self._repo = repo
         self._rev = rev or 'patch'
         self._status = [[], [], []]
@@ -38,7 +42,6 @@ class patchctx(object):
         self._desc = ''
         self._branch = ''
         self._node = node.nullid
-        self._identity = node.nullid
         self._mtime = None
         self._fsize = 0
         self._parseerror = None
@@ -49,9 +52,6 @@ class patchctx(object):
             self._fsize = os.path.getsize(patchpath)
             ph = mq.patchheader(self._path)
             self._ph = ph
-            hash = util.sha1(self._path)
-            hash.update(str(self._mtime))
-            self._identity = hash.digest()
         except EnvironmentError:
             self._date = util.makedate()
             return
@@ -148,12 +148,10 @@ class patchctx(object):
     def thgmqpatchname(self):       return self._patchname
     def thgbranchhead(self):        return False
     def thgmqunappliedpatch(self):  return True
-    def thgid(self):                return self._identity
 
     # largefiles/kbfiles methods
     def hasStandin(self, file):     return False
     def isStandin(self, path):      return False
-    def removeStandin(self, path):  return path
 
     def longsummary(self):
         if self._repo.ui.configbool('tortoisehg', 'longsummary'):
@@ -171,6 +169,8 @@ class patchctx(object):
 
     def thgmqoriginalparent(self):
         '''The revision id of the original patch parent'''
+        if not util.safehasattr(self, '_ph'):
+            return ''
         return self._ph.parent
 
     def thgmqpatchdata(self, wfile):
